# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.issuing._transaction import Transaction
    from stripe.params.test_helpers.issuing._transaction_create_force_capture_params import (
        TransactionCreateForceCaptureParams,
    )
    from stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params import (
        TransactionCreateUnlinkedRefundParams,
    )
    from stripe.params.test_helpers.issuing._transaction_refund_params import (
        TransactionRefundParams,
    )


class TransactionService(StripeService):
    def refund(
        self,
        transaction: str,
        params: Optional["TransactionRefundParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Refund a test-mode Transaction.
        """
        return cast(
            "Transaction",
            self._request(
                "post",
                "/v1/test_helpers/issuing/transactions/{transaction}/refund".format(
                    transaction=sanitize_id(transaction),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def refund_async(
        self,
        transaction: str,
        params: Optional["TransactionRefundParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Refund a test-mode Transaction.
        """
        return cast(
            "Transaction",
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/transactions/{transaction}/refund".format(
                    transaction=sanitize_id(transaction),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create_force_capture(
        self,
        params: "TransactionCreateForceCaptureParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Allows the user to capture an arbitrary amount, also known as a forced capture.
        """
        return cast(
            "Transaction",
            self._request(
                "post",
                "/v1/test_helpers/issuing/transactions/create_force_capture",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_force_capture_async(
        self,
        params: "TransactionCreateForceCaptureParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Allows the user to capture an arbitrary amount, also known as a forced capture.
        """
        return cast(
            "Transaction",
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/transactions/create_force_capture",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create_unlinked_refund(
        self,
        params: "TransactionCreateUnlinkedRefundParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Allows the user to refund an arbitrary amount, also known as a unlinked refund.
        """
        return cast(
            "Transaction",
            self._request(
                "post",
                "/v1/test_helpers/issuing/transactions/create_unlinked_refund",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_unlinked_refund_async(
        self,
        params: "TransactionCreateUnlinkedRefundParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Transaction":
        """
        Allows the user to refund an arbitrary amount, also known as a unlinked refund.
        """
        return cast(
            "Transaction",
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/transactions/create_unlinked_refund",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._file import File
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._file_create_params import FileCreateParams
    from stripe.params._file_list_params import FileListParams
    from stripe.params._file_retrieve_params import FileRetrieveParams


class FileService(StripeService):
    def list(
        self,
        params: Optional["FileListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[File]":
        """
        Returns a list of the files that your account has access to. Stripe sorts and returns the files by their creation dates, placing the most recently created files at the top.
        """
        return cast(
            "ListObject[File]",
            self._request(
                "get",
                "/v1/files",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["FileListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[File]":
        """
        Returns a list of the files that your account has access to. Stripe sorts and returns the files by their creation dates, placing the most recently created files at the top.
        """
        return cast(
            "ListObject[File]",
            await self._request_async(
                "get",
                "/v1/files",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "FileCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "File":
        """
        To upload a file to Stripe, you need to send a request of type multipart/form-data. Include the file you want to upload in the request, and the parameters for creating a file.

        All of Stripe's officially supported Client libraries support sending multipart/form-data.
        """
        if options is None:
            options = {}
        options["content_type"] = "multipart/form-data"
        return cast(
            "File",
            self._request(
                "post",
                "/v1/files",
                base_address="files",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "FileCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "File":
        """
        To upload a file to Stripe, you need to send a request of type multipart/form-data. Include the file you want to upload in the request, and the parameters for creating a file.

        All of Stripe's officially supported Client libraries support sending multipart/form-data.
        """
        if options is None:
            options = {}
        options["content_type"] = "multipart/form-data"
        return cast(
            "File",
            await self._request_async(
                "post",
                "/v1/files",
                base_address="files",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        file: str,
        params: Optional["FileRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "File":
        """
        Retrieves the details of an existing file object. After you supply a unique file ID, Stripe returns the corresponding file object. Learn how to [access file contents](https://docs.stripe.com/docs/file-upload#download-file-contents).
        """
        return cast(
            "File",
            self._request(
                "get",
                "/v1/files/{file}".format(file=sanitize_id(file)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        file: str,
        params: Optional["FileRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "File":
        """
        Retrieves the details of an existing file object. After you supply a unique file ID, Stripe returns the corresponding file object. Learn how to [access file contents](https://docs.stripe.com/docs/file-upload#download-file-contents).
        """
        return cast(
            "File",
            await self._request_async(
                "get",
                "/v1/files/{file}".format(file=sanitize_id(file)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class ApiKeyCreateParams(TypedDict):
    connect_permissions: NotRequired[List[str]]
    """
    List of connect permissions for this API key.
    """
    expires_at: NotRequired[str]
    """
    Timestamp at which the key expires. If not provided, the key never expires.
    """
    name: NotRequired[str]
    """
    Name for the API key.
    """
    note: NotRequired[str]
    """
    Note or description for the API key.
    """
    permissions: NotRequired[List[str]]
    """
    List of permissions for this API key.
    """
    public_key: NotRequired["ApiKeyCreateParamsPublicKey"]
    """
    Public key for encrypting the API key secret.
    This must a PEM-formatted RSA key suitable for encryption, >= 2048 bits.
    A public key is required when creating secret keys.
    Publishable keys are not encrypted and a public key should not be included.
    """
    type: Literal["publishable_key", "secret_key"]
    """
    Type of the API key to create (secret or publishable).
    """


class ApiKeyCreateParamsPublicKey(TypedDict):
    id: NotRequired[str]
    """
    Caller's identifier of the public key. This is used for tracking purposes only, and will be echoed in the response if provided.
    """
    pem_key: NotRequired["ApiKeyCreateParamsPublicKeyPemKey"]
    """
    PEM-formatted public key.
    """


class ApiKeyCreateParamsPublicKeyPemKey(TypedDict):
    algorithm: str
    """
    The encryption algorithm used with this key (e.g., RSA).
    """
    data: str
    """
    The PEM-encoded public key data. Newlines are required between header/footer and body, and optional within the body.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_create_params import (
        CustomPricingUnitOverageRateCreateParams,
    )
    from stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_delete_params import (
        CustomPricingUnitOverageRateDeleteParams,
    )
    from stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_list_params import (
        CustomPricingUnitOverageRateListParams,
    )
    from stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_retrieve_params import (
        CustomPricingUnitOverageRateRetrieveParams,
    )
    from stripe.v2._deleted_object import DeletedObject
    from stripe.v2._list_object import ListObject
    from stripe.v2.billing._rate_card_custom_pricing_unit_overage_rate import (
        RateCardCustomPricingUnitOverageRate,
    )


class CustomPricingUnitOverageRateService(StripeService):
    def list(
        self,
        rate_card_id: str,
        params: Optional["CustomPricingUnitOverageRateListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[RateCardCustomPricingUnitOverageRate]":
        """
        List all Rate Card Custom Pricing Unit Overage Rates on a Rate Card.
        """
        return cast(
            "ListObject[RateCardCustomPricingUnitOverageRate]",
            self._request(
                "get",
                "/v2/billing/rate_cards/{rate_card_id}/custom_pricing_unit_overage_rates".format(
                    rate_card_id=sanitize_id(rate_card_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        rate_card_id: str,
        params: Optional["CustomPricingUnitOverageRateListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[RateCardCustomPricingUnitOverageRate]":
        """
        List all Rate Card Custom Pricing Unit Overage Rates on a Rate Card.
        """
        return cast(
            "ListObject[RateCardCustomPricingUnitOverageRate]",
            await self._request_async(
                "get",
                "/v2/billing/rate_cards/{rate_card_id}/custom_pricing_unit_overage_rates".format(
                    rate_card_id=sanitize_id(rate_card_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        rate_card_id: str,
        params: "CustomPricingUnitOverageRateCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "RateCardCustomPricingUnitOverageRate":
        """
        Create a Rate Card Custom Pricing Unit Overage Rate on a Rate Card.
        """
        return cast(
            "RateCardCustomPricingUnitOverageRate",
            self._request(
                "post",
                "/v2/billing/rate_cards/{rate_card_id}/custom_pricing_unit_overage_rates".format(
                    rate_card_id=sanitize_id(rate_card_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        rate_card_id: str,
        params: "CustomPricingUnitOverageRateCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "RateCardCustomPricingUnitOverageRate":
        """
        Create a Rate Card Custom Pricing Unit Overage Rate on a Rate Card.
        """
        return cast(
            "RateCardCustomPricingUnitOverageRate",
            await self._request_async(
                "post",
                "/v2/billing/rate_cards/{rate_card_id}/custom_pricing_unit_overage_rates".format(
                    rate_card_id=sanitize_id(rate_card_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def delete(
        self,
        rate_card_id: str,
        id: str,
        params: Optional["CustomPricingUnitOverageRateDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "DeletedObject":
        """
        Delete a Rate Card Custom Pricing Unit Overage Rate from a Rate Card.
        """
        return cast(
            "DeletedObject",
            self._request(
                "delete",
                "/v2/billing/rate_cards/{rate_card_id}/custom_pricing_unit_overage_rates/{id}".format(
                    rate_card_id=sanitize_id(rate_card_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        rate_card_id: str,
        id: str,
        params: Optional["CustomPricingUnitOverageRateDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "DeletedObject":
        """
        Delete a Rate Card Custom Pricing Unit Overage Rate from a Rate Card.
        """
        return cast(
            "DeletedObject",
            await self._request_async(
                "delete",
                "/v2/billing/rate_cards/{rate_card_id}/custom_pricing_unit_overage_rates/{id}".format(
                    rate_card_id=sanitize_id(rate_card_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        rate_card_id: str,
        id: str,
        params: Optional["CustomPricingUnitOverageRateRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RateCardCustomPricingUnitOverageRate":
        """
        Retrieve a Rate Card Custom Pricing Unit Overage Rate from a Rate Card.
        """
        return cast(
            "RateCardCustomPricingUnitOverageRate",
            self._request(
                "get",
                "/v2/billing/rate_cards/{rate_card_id}/custom_pricing_unit_overage_rates/{id}".format(
                    rate_card_id=sanitize_id(rate_card_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        rate_card_id: str,
        id: str,
        params: Optional["CustomPricingUnitOverageRateRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "RateCardCustomPricingUnitOverageRate":
        """
        Retrieve a Rate Card Custom Pricing Unit Overage Rate from a Rate Card.
        """
        return cast(
            "RateCardCustomPricingUnitOverageRate",
            await self._request_async(
                "get",
                "/v2/billing/rate_cards/{rate_card_id}/custom_pricing_unit_overage_rates/{id}".format(
                    rate_card_id=sanitize_id(rate_card_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.terminal._configuration_service import ConfigurationService
    from stripe.terminal._connection_token_service import (
        ConnectionTokenService,
    )
    from stripe.terminal._location_service import LocationService
    from stripe.terminal._onboarding_link_service import OnboardingLinkService
    from stripe.terminal._reader_collected_data_service import (
        ReaderCollectedDataService,
    )
    from stripe.terminal._reader_service import ReaderService

_subservices = {
    "configurations": [
        "stripe.terminal._configuration_service",
        "ConfigurationService",
    ],
    "connection_tokens": [
        "stripe.terminal._connection_token_service",
        "ConnectionTokenService",
    ],
    "locations": ["stripe.terminal._location_service", "LocationService"],
    "onboarding_links": [
        "stripe.terminal._onboarding_link_service",
        "OnboardingLinkService",
    ],
    "readers": ["stripe.terminal._reader_service", "ReaderService"],
    "reader_collected_data": [
        "stripe.terminal._reader_collected_data_service",
        "ReaderCollectedDataService",
    ],
}


class TerminalService(StripeService):
    configurations: "ConfigurationService"
    connection_tokens: "ConnectionTokenService"
    locations: "LocationService"
    onboarding_links: "OnboardingLinkService"
    readers: "ReaderService"
    reader_collected_data: "ReaderCollectedDataService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class OutboundPayment(StripeObject):
    """
    OutboundPayment represents a single money movement from one FinancialAccount you own to a payout method someone else owns.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.outbound_payment"]] = (
        "v2.money_management.outbound_payment"
    )

    class DeliveryOptions(StripeObject):
        class PaperCheck(StripeObject):
            attachment: Optional[str]
            """
            The ID of a file to include as an attachment with the paper check.
            """
            memo: str
            """
            Memo printed on the memo field of the check.
            """
            shipping_speed: Literal["priority", "standard"]
            """
            Open Enum. Shipping speed of the paper check.
            """
            signature: str
            """
            Signature for the paper check.
            """

        bank_account: Optional[Literal["automatic", "local", "wire"]]
        """
        Open Enum. Method for bank account.
        """
        paper_check: Optional[PaperCheck]
        """
        Delivery options for paper check.
        """
        speed: Optional[Literal["instant", "next_business_day", "standard"]]
        """
        Open Enum. Speed of the payout.
        """
        _inner_class_types = {"paper_check": PaperCheck}

    class From(StripeObject):
        debited: Amount
        """
        The monetary amount debited from the sender, only set on responses.
        """
        financial_account: str
        """
        The FinancialAccount that funds were pulled from.
        """

    class RecipientNotification(StripeObject):
        setting: Literal["configured", "none"]
        """
        Closed Enum. Configuration option to enable or disable notifications to recipients.
        Do not send notifications when setting is NONE. Default to account setting when setting is CONFIGURED or not set.
        """

    class StatusDetails(StripeObject):
        class Failed(StripeObject):
            reason: Literal[
                "fx_rate_drift_exceeded_after_review",
                "paper_check_attachment_too_large",
                "paper_check_expired",
                "paper_check_undeliverable",
                "payout_method_amount_limit_exceeded",
                "payout_method_declined",
                "payout_method_does_not_exist",
                "payout_method_expired",
                "payout_method_unsupported",
                "payout_method_usage_frequency_limit_exceeded",
                "review_rejected",
                "unknown_failure",
            ]
            """
            Open Enum. The `failed` status reason.
            """

        class Processing(StripeObject):
            reason: Literal["under_review"]
            """
            Open Enum. The `processing` status reason.
            """

        class Returned(StripeObject):
            reason: Literal[
                "payout_method_canceled_by_customer",
                "payout_method_closed",
                "payout_method_currency_unsupported",
                "payout_method_does_not_exist",
                "payout_method_holder_address_incorrect",
                "payout_method_holder_details_incorrect",
                "payout_method_holder_name_incorrect",
                "payout_method_invalid_account_number",
                "payout_method_restricted",
                "recalled",
                "unknown_failure",
            ]
            """
            Open Enum. The `returned` status reason.
            """

        failed: Optional[Failed]
        """
        The `failed` status reason.
        """
        processing: Optional[Processing]
        """
        The `processing` status details.
        """
        returned: Optional[Returned]
        """
        The `returned` status reason.
        """
        _inner_class_types = {
            "failed": Failed,
            "processing": Processing,
            "returned": Returned,
        }

    class StatusTransitions(StripeObject):
        canceled_at: Optional[str]
        """
        Timestamp describing when an OutboundPayment changed status to `canceled`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        failed_at: Optional[str]
        """
        Timestamp describing when an OutboundPayment changed status to `failed`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        posted_at: Optional[str]
        """
        Timestamp describing when an OutboundPayment changed status to `posted`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        returned_at: Optional[str]
        """
        Timestamp describing when an OutboundPayment changed status to `returned`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """

    class To(StripeObject):
        class PayoutMethodOptions(StripeObject):
            class BankAccount(StripeObject):
                class PreferredNetworkOptions(StripeObject):
                    class Ach(StripeObject):
                        submission: Optional[Literal["next_day", "same_day"]]
                        """
                        Open Enum. ACH submission timing.
                        """
                        transaction_purpose: Optional[Literal["payroll"]]
                        """
                        The transaction purpose for this ACH payment.
                        """

                    ach: Optional[Ach]
                    """
                    ACH-specific network options.
                    """
                    _inner_class_types = {"ach": Ach}

                preferred_network_options: Optional[PreferredNetworkOptions]
                """
                Per-network configuration options.
                """
                preferred_networks: List[
                    Literal[
                        "ach",
                        "becs",
                        "eft",
                        "fedwire",
                        "fps",
                        "npp",
                        "rtp",
                        "sepa_credit",
                        "sepa_instant",
                        "swift",
                    ]
                ]
                """
                The preferred networks to use for this OutboundPayment.
                """
                _inner_class_types = {
                    "preferred_network_options": PreferredNetworkOptions,
                }

            bank_account: Optional[BankAccount]
            """
            Options for bank account payout methods.
            """
            _inner_class_types = {"bank_account": BankAccount}

        credited: Amount
        """
        The monetary amount being credited to the destination.
        """
        payout_method: str
        """
        The payout method which the OutboundPayment uses to send payout.
        """
        payout_method_options: Optional[PayoutMethodOptions]
        """
        Payout method options for the OutboundPayment.
        """
        recipient: str
        """
        To which account the OutboundPayment is sent.
        """
        _inner_class_types = {"payout_method_options": PayoutMethodOptions}

    class TraceId(StripeObject):
        status: Literal["pending", "supported", "unsupported"]
        """
        Possible values are `pending`, `supported`, and `unsupported`. Initially set to `pending`, it changes to
        `supported` when the recipient bank provides a trace ID, or `unsupported` if the recipient bank doesn't support it.
        Note that this status may not align with the OutboundPayment or OutboundTransfer status and can remain `pending`
        even after the payment or transfer is posted.
        """
        value: Optional[str]
        """
        The trace ID value if `trace_id.status` is `supported`, otherwise empty.
        """

    class TrackingDetails(StripeObject):
        class PaperCheck(StripeObject):
            class MailingAddress(StripeObject):
                city: Optional[str]
                """
                City, district, suburb, town, or village.
                """
                country: Optional[str]
                """
                Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                """
                line1: Optional[str]
                """
                Address line 1 (e.g., street, PO Box, or company name).
                """
                line2: Optional[str]
                """
                Address line 2 (e.g., apartment, suite, unit, or building).
                """
                postal_code: Optional[str]
                """
                ZIP or postal code.
                """
                state: Optional[str]
                """
                State, county, province, or region.
                """

            carrier: Literal["fedex", "usps"]
            """
            Open Enum. Carrier of the paper check.
            """
            check_number: str
            """
            Check number.
            """
            current_postal_code: str
            """
            Postal code of the latest tracking update.
            """
            mailing_address: MailingAddress
            """
            Mailing address of the paper check.
            """
            tracking_number: str
            """
            Tracking number for the check.
            """
            tracking_status: Literal["delivered", "in_transit", "mailed"]
            """
            Open Enum. Tracking status of the paper check.
            """
            updated_at: str
            """
            When the tracking details were last updated.
            """
            _inner_class_types = {"mailing_address": MailingAddress}

        paper_check: Optional[PaperCheck]
        """
        Paper check tracking details.
        """
        _inner_class_types = {"paper_check": PaperCheck}

    amount: Amount
    """
    The "presentment amount" for the OutboundPayment.
    """
    cancelable: bool
    """
    Returns true if the OutboundPayment can be canceled, and false otherwise.
    """
    created: str
    """
    Time at which the OutboundPayment was created.
    Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    delivery_options: Optional[DeliveryOptions]
    """
    Delivery options to be used to send the OutboundPayment.
    """
    description: Optional[str]
    """
    An arbitrary string attached to the OutboundPayment. Often useful for displaying to users.
    """
    expected_arrival_date: Optional[str]
    """
    The date when funds are expected to arrive in the payout method. This field is not set if the payout method is in a `failed`, `canceled`, or `returned` state.
    Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    from_: From
    """
    The FinancialAccount that funds were pulled from.
    """
    id: str
    """
    Unique identifier for the OutboundPayment.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["v2.money_management.outbound_payment"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    outbound_payment_quote: Optional[str]
    """
    The quote for this OutboundPayment. Only required for countries with regulatory mandates to display fee estimates before OutboundPayment creation.
    """
    purpose: Optional[Literal["payroll"]]
    """
    The purpose of the OutboundPayment.
    """
    receipt_url: Optional[str]
    """
    A link to the Stripe-hosted receipt for this OutboundPayment. The receipt link remains active for 60 days from the OutboundPayment creation date. After this period, the link will expire and the receipt url value will be null.
    """
    recipient_notification: RecipientNotification
    """
    Details about the OutboundPayment notification settings for recipient.
    """
    recipient_verification: Optional[str]
    """
    The recipient verification id for this OutboundPayment. Only required for countries with regulatory mandates to verify recipient names before OutboundPayment creation.
    """
    statement_descriptor: str
    """
    The description that appears on the receiving end for an OutboundPayment (for example, bank statement for external bank transfer). It will default to `STRIPE` if not set on the account settings.
    """
    status: Literal["canceled", "failed", "posted", "processing", "returned"]
    """
    Closed Enum. Current status of the OutboundPayment: `processing`, `failed`, `posted`, `returned`, `canceled`.
    An OutboundPayment is `processing` if it has been created and is processing.
    The status changes to `posted` once the OutboundPayment has been "confirmed" and funds have left the account, or to `failed` or `canceled`.
    If an OutboundPayment fails to arrive at its payout method, its status will change to `returned`.
    """
    status_details: Optional[StatusDetails]
    """
    Status details for an OutboundPayment in a `processing`, `failed`, or `returned` state.
    """
    status_transitions: Optional[StatusTransitions]
    """
    Hash containing timestamps of when the object transitioned to a particular status.
    """
    to: To
    """
    To which payout method the OutboundPayment was sent.
    """
    trace_id: TraceId
    """
    A unique identifier that can be used to track this OutboundPayment with recipient bank. Banks might call this a "reference number" or something similar.
    """
    tracking_details: Optional[TrackingDetails]
    """
    Information to track this OutboundPayment with the recipient bank.
    """
    _inner_class_types = {
        "delivery_options": DeliveryOptions,
        "from": From,
        "recipient_notification": RecipientNotification,
        "status_details": StatusDetails,
        "status_transitions": StatusTransitions,
        "to": To,
        "trace_id": TraceId,
        "tracking_details": TrackingDetails,
    }
    _field_remappings = {"from_": "from"}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class MeterEventSummary(StripeObject):
    """
    A billing meter event summary represents an aggregated view of a customer's billing meter events within a specified timeframe. It indicates how much
    usage was accrued by a customer for that period.

    Note: Meters events are aggregated asynchronously so the meter event summaries provide an eventually consistent view of the reported usage.
    """

    OBJECT_NAME: ClassVar[Literal["billing.meter_event_summary"]] = (
        "billing.meter_event_summary"
    )
    aggregated_value: float
    """
    Aggregated value of all the events within `start_time` (inclusive) and `end_time` (inclusive). The aggregation strategy is defined on meter via `default_aggregation`.
    """
    dimensions: Optional[UntypedStripeObject[str]]
    """
    Key-value pairs of dimension values for event summaries with grouping on dimensions.
    """
    end_time: int
    """
    End timestamp for this event summary (exclusive). Must be aligned with minute boundaries.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    meter: str
    """
    The meter associated with this event summary.
    """
    object: Literal["billing.meter_event_summary"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    start_time: int
    """
    Start timestamp for this event summary (inclusive). Must be aligned with minute boundaries.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class CurrencyConversionListParams(TypedDict):
    financial_account: NotRequired[str]
    """
    The ID of the FinancialAccount to filter by.
    """
    limit: NotRequired[int]
    """
    The page limit.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
import json
from stripe._api_version import _ApiVersion
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from uuid import uuid4
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._promotion_code import PromotionCode
    from stripe._request_options import RequestOptions
    from stripe.params._promotion_code_create_params import (
        PromotionCodeCreateParams,
    )
    from stripe.params._promotion_code_list_params import (
        PromotionCodeListParams,
    )
    from stripe.params._promotion_code_retrieve_params import (
        PromotionCodeRetrieveParams,
    )
    from stripe.params._promotion_code_update_params import (
        PromotionCodeUpdateParams,
    )


class PromotionCodeService(StripeService):
    def list(
        self,
        params: Optional["PromotionCodeListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PromotionCode]":
        """
        Returns a list of your promotion codes.
        """
        return cast(
            "ListObject[PromotionCode]",
            self._request(
                "get",
                "/v1/promotion_codes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["PromotionCodeListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PromotionCode]":
        """
        Returns a list of your promotion codes.
        """
        return cast(
            "ListObject[PromotionCode]",
            await self._request_async(
                "get",
                "/v1/promotion_codes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "PromotionCodeCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PromotionCode":
        """
        A promotion code points to an underlying promotion. You can optionally restrict the code to a specific customer, redemption limit, and expiration date.
        """
        return cast(
            "PromotionCode",
            self._request(
                "post",
                "/v1/promotion_codes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "PromotionCodeCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PromotionCode":
        """
        A promotion code points to an underlying promotion. You can optionally restrict the code to a specific customer, redemption limit, and expiration date.
        """
        return cast(
            "PromotionCode",
            await self._request_async(
                "post",
                "/v1/promotion_codes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        promotion_code: str,
        params: Optional["PromotionCodeRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PromotionCode":
        """
        Retrieves the promotion code with the given ID. In order to retrieve a promotion code by the customer-facing code use [list](https://docs.stripe.com/docs/api/promotion_codes/list) with the desired code.
        """
        return cast(
            "PromotionCode",
            self._request(
                "get",
                "/v1/promotion_codes/{promotion_code}".format(
                    promotion_code=sanitize_id(promotion_code),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        promotion_code: str,
        params: Optional["PromotionCodeRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PromotionCode":
        """
        Retrieves the promotion code with the given ID. In order to retrieve a promotion code by the customer-facing code use [list](https://docs.stripe.com/docs/api/promotion_codes/list) with the desired code.
        """
        return cast(
            "PromotionCode",
            await self._request_async(
                "get",
                "/v1/promotion_codes/{promotion_code}".format(
                    promotion_code=sanitize_id(promotion_code),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        promotion_code: str,
        params: Optional["PromotionCodeUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PromotionCode":
        """
        Updates the specified promotion code by setting the values of the parameters passed. Most fields are, by design, not editable.
        """
        return cast(
            "PromotionCode",
            self._request(
                "post",
                "/v1/promotion_codes/{promotion_code}".format(
                    promotion_code=sanitize_id(promotion_code),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        promotion_code: str,
        params: Optional["PromotionCodeUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PromotionCode":
        """
        Updates the specified promotion code by setting the values of the parameters passed. Most fields are, by design, not editable.
        """
        return cast(
            "PromotionCode",
            await self._request_async(
                "post",
                "/v1/promotion_codes/{promotion_code}".format(
                    promotion_code=sanitize_id(promotion_code),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def serialize_batch_create(
        self,
        params: Optional["PromotionCodeCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a PromotionCode create request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)

    def serialize_batch_update(
        self,
        promotion_code: str,
        params: Optional["PromotionCodeUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a PromotionCode update request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {"promotion_code": promotion_code},
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._payment_method import PaymentMethod
    from stripe._request_options import RequestOptions
    from stripe.params._customer_payment_method_list_params import (
        CustomerPaymentMethodListParams,
    )
    from stripe.params._customer_payment_method_retrieve_params import (
        CustomerPaymentMethodRetrieveParams,
    )


class CustomerPaymentMethodService(StripeService):
    def list(
        self,
        customer: str,
        params: Optional["CustomerPaymentMethodListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PaymentMethod]":
        """
        Returns a list of PaymentMethods for a given Customer
        """
        return cast(
            "ListObject[PaymentMethod]",
            self._request(
                "get",
                "/v1/customers/{customer}/payment_methods".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        customer: str,
        params: Optional["CustomerPaymentMethodListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PaymentMethod]":
        """
        Returns a list of PaymentMethods for a given Customer
        """
        return cast(
            "ListObject[PaymentMethod]",
            await self._request_async(
                "get",
                "/v1/customers/{customer}/payment_methods".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        customer: str,
        payment_method: str,
        params: Optional["CustomerPaymentMethodRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentMethod":
        """
        Retrieves a PaymentMethod object for a given Customer.
        """
        return cast(
            "PaymentMethod",
            self._request(
                "get",
                "/v1/customers/{customer}/payment_methods/{payment_method}".format(
                    customer=sanitize_id(customer),
                    payment_method=sanitize_id(payment_method),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        customer: str,
        payment_method: str,
        params: Optional["CustomerPaymentMethodRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentMethod":
        """
        Retrieves a PaymentMethod object for a given Customer.
        """
        return cast(
            "PaymentMethod",
            await self._request_async(
                "get",
                "/v1/customers/{customer}/payment_methods/{payment_method}".format(
                    customer=sanitize_id(customer),
                    payment_method=sanitize_id(payment_method),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict
from typing_extensions import Literal, NotRequired, TypedDict


class PricingPlanCreateParams(TypedDict):
    currency: str
    """
    The currency of the PricingPlan.
    """
    description: NotRequired[str]
    """
    Description of the pricing plan. Maximum length of 500 characters.
    """
    display_name: str
    """
    Display name of the PricingPlan. Maximum 250 characters.
    """
    lookup_key: NotRequired[str]
    """
    An internal key you can use to search for a particular PricingPlan. Maximum length of 200 characters.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    tax_behavior: Literal["exclusive", "inclusive"]
    """
    The tax behavior for Stripe Tax — whether the pricing plan price includes or excludes tax.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.billing._custom_pricing_unit_create_params import (
        CustomPricingUnitCreateParams,
    )
    from stripe.params.v2.billing._custom_pricing_unit_list_params import (
        CustomPricingUnitListParams,
    )
    from stripe.params.v2.billing._custom_pricing_unit_retrieve_params import (
        CustomPricingUnitRetrieveParams,
    )
    from stripe.params.v2.billing._custom_pricing_unit_update_params import (
        CustomPricingUnitUpdateParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.billing._custom_pricing_unit import CustomPricingUnit


class CustomPricingUnitService(StripeService):
    def list(
        self,
        params: Optional["CustomPricingUnitListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[CustomPricingUnit]":
        """
        List all Custom Pricing Unit objects.
        """
        return cast(
            "ListObject[CustomPricingUnit]",
            self._request(
                "get",
                "/v2/billing/custom_pricing_units",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["CustomPricingUnitListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[CustomPricingUnit]":
        """
        List all Custom Pricing Unit objects.
        """
        return cast(
            "ListObject[CustomPricingUnit]",
            await self._request_async(
                "get",
                "/v2/billing/custom_pricing_units",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "CustomPricingUnitCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CustomPricingUnit":
        """
        Create a Custom Pricing Unit object.
        """
        return cast(
            "CustomPricingUnit",
            self._request(
                "post",
                "/v2/billing/custom_pricing_units",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "CustomPricingUnitCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CustomPricingUnit":
        """
        Create a Custom Pricing Unit object.
        """
        return cast(
            "CustomPricingUnit",
            await self._request_async(
                "post",
                "/v2/billing/custom_pricing_units",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["CustomPricingUnitRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CustomPricingUnit":
        """
        Retrieve a Custom Pricing Unit object.
        """
        return cast(
            "CustomPricingUnit",
            self._request(
                "get",
                "/v2/billing/custom_pricing_units/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["CustomPricingUnitRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CustomPricingUnit":
        """
        Retrieve a Custom Pricing Unit object.
        """
        return cast(
            "CustomPricingUnit",
            await self._request_async(
                "get",
                "/v2/billing/custom_pricing_units/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["CustomPricingUnitUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CustomPricingUnit":
        """
        Update a Custom Pricing Unit object.
        """
        return cast(
            "CustomPricingUnit",
            self._request(
                "post",
                "/v2/billing/custom_pricing_units/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["CustomPricingUnitUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CustomPricingUnit":
        """
        Update a Custom Pricing Unit object.
        """
        return cast(
            "CustomPricingUnit",
            await self._request_async(
                "post",
                "/v2/billing/custom_pricing_units/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class ServiceActionUpdateParams(TypedDict):
    credit_grant: NotRequired["ServiceActionUpdateParamsCreditGrant"]
    """
    Details for the credit grant. Can only be set if the service action's `type` is `credit_grant`.
    """
    credit_grant_per_tenant: NotRequired[
        "ServiceActionUpdateParamsCreditGrantPerTenant"
    ]
    """
    Details for the credit grant per tenant. Can only be set if the service action's `type` is `credit_grant_per_tenant`.
    """
    lookup_key: NotRequired[str]
    """
    An internal key you can use to search for this service action. Maximum length of 200 characters.
    """


class ServiceActionUpdateParamsCreditGrant(TypedDict):
    name: NotRequired[str]
    """
    A descriptive name shown in dashboard.
    """


class ServiceActionUpdateParamsCreditGrantPerTenant(TypedDict):
    name: NotRequired[str]
    """
    A descriptive name shown in dashboard.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar
from typing_extensions import Literal


class AccountInferredBalance(StripeObject):
    """
    A historical balance for the account on a particular day. It may be sourced from a balance snapshot provided by a financial institution, or inferred using transactions data.
    """

    OBJECT_NAME: ClassVar[
        Literal["financial_connections.account_inferred_balance"]
    ] = "financial_connections.account_inferred_balance"
    as_of: int
    """
    The time for which this balance was calculated, measured in seconds since the Unix epoch. If the balance was computed by Stripe and not provided directly by a financial institution, it will always be 23:59:59 UTC.
    """
    current: UntypedStripeObject[int]
    """
    The balances owed to (or by) the account holder, before subtracting any outbound pending transactions or adding any inbound pending transactions.

    Each key is a three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase.

    Each value is a integer amount. A positive amount indicates money owed to the account holder. A negative amount indicates money owed by the account holder.
    """
    id: str
    """
    Unique identifier for the object.
    """
    object: Literal["financial_connections.account_inferred_balance"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2.commerce import product_catalog as product_catalog
    from stripe.v2.commerce._product_catalog_import import (
        ProductCatalogImport as ProductCatalogImport,
    )
    from stripe.v2.commerce._product_catalog_service import (
        ProductCatalogService as ProductCatalogService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "product_catalog": ("stripe.v2.commerce.product_catalog", True),
    "ProductCatalogImport": (
        "stripe.v2.commerce._product_catalog_import",
        False,
    ),
    "ProductCatalogService": (
        "stripe.v2.commerce._product_catalog_service",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._balance_transaction import BalanceTransaction
    from stripe.params._balance_transfer_create_params import (
        BalanceTransferCreateParams,
    )


class BalanceTransfer(CreateableAPIResource["BalanceTransfer"]):
    """
    Balance transfers represent funds moving between balance types on your Stripe account.
    They currently support moving funds between your Stripe balance and your [Issuing](https://docs.stripe.com/issuing) balance and between your [Allocated Funds](https://docs.stripe.com/connect/funds-segregation) balance and your Stripe balance.
    """

    OBJECT_NAME: ClassVar[Literal["balance_transfer"]] = "balance_transfer"

    class DestinationBalance(StripeObject):
        class Issuing(StripeObject):
            balance_transaction: Optional[
                ExpandableField["BalanceTransaction"]
            ]
            """
            Identifier for the balance_transaction that increased the destination balance.
            """

        class Payments(StripeObject):
            balance_transaction: Optional[
                ExpandableField["BalanceTransaction"]
            ]
            """
            Identifier for the balance_transaction that increased the destination balance.
            """

        issuing: Optional[Issuing]
        payments: Optional[Payments]
        type: str
        """
        Destination balance type to adjust for the Balance Transfer. One of `payments`, `issuing`, or `allocated_funds`.
        """
        _inner_class_types = {"issuing": Issuing, "payments": Payments}

    class SourceBalance(StripeObject):
        class Issuing(StripeObject):
            balance_transaction: Optional[
                ExpandableField["BalanceTransaction"]
            ]
            """
            Identifier for the balance_transaction that decreased the source balance.
            """

        class Payments(StripeObject):
            balance_transaction: ExpandableField["BalanceTransaction"]
            """
            Identifier for the balance_transaction that decreased the source balance.
            """
            source_type: Optional[str]
            """
            The payments balance type that this BalanceTransfer pulled funds from. One of `card`, `fpx`, or `bank_account`.
            """

        issuing: Optional[Issuing]
        payments: Optional[Payments]
        type: str
        """
        Source balance type to adjust for the Balance Transfer. One of `payments`, `issuing`, or `allocated_funds`.
        """
        _inner_class_types = {"issuing": Issuing, "payments": Payments}

    amount: int
    """
    A positive integer representing how much was transferred in the smallest currency unit.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    destination_balance: Optional[DestinationBalance]
    """
    The balance that funds were transferred to.
    """
    hosted_regulatory_receipt_url: Optional[str]
    """
    A [hosted transaction receipt](https://docs.stripe.com/treasury/moving-money/regulatory-receipts) URL that is provided when money movement is considered regulated under Stripe's money transmission licenses.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: UntypedStripeObject[str]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["balance_transfer"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    source_balance: Optional[SourceBalance]
    """
    The balance that funds were transferred from. One of `card`, `fpx`, or `bank_account`.
    """

    @classmethod
    def create(
        cls, **params: Unpack["BalanceTransferCreateParams"]
    ) -> "BalanceTransfer":
        """
        Creates a balance transfer. For Issuing use cases, funds will be debited immediately from the source balance and credited to the destination balance immediately (if your account is based in the US) or next-business-day (if your account is based in the EU). For Segregated Separate Charges and Transfers use cases, funds will be debited immediately from the source balance and credited immediately to the destination balance.
        """
        return cast(
            "BalanceTransfer",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["BalanceTransferCreateParams"]
    ) -> "BalanceTransfer":
        """
        Creates a balance transfer. For Issuing use cases, funds will be debited immediately from the source balance and credited to the destination balance immediately (if your account is based in the US) or next-business-day (if your account is based in the EU). For Segregated Separate Charges and Transfers use cases, funds will be debited immediately from the source balance and credited immediately to the destination balance.
        """
        return cast(
            "BalanceTransfer",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    _inner_class_types = {
        "destination_balance": DestinationBalance,
        "source_balance": SourceBalance,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.orchestration._payment_attempt import PaymentAttempt
    from stripe.params.orchestration._payment_attempt_retrieve_params import (
        PaymentAttemptRetrieveParams,
    )


class PaymentAttemptService(StripeService):
    def retrieve(
        self,
        payment_attempt_record: str,
        params: Optional["PaymentAttemptRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttempt":
        """
        Retrieves orchestration information for the given payment attempt record (e.g. return url).
        """
        return cast(
            "PaymentAttempt",
            self._request(
                "get",
                "/v1/orchestration/payment_attempts/{payment_attempt_record}".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        payment_attempt_record: str,
        params: Optional["PaymentAttemptRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttempt":
        """
        Retrieves orchestration information for the given payment attempt record (e.g. return url).
        """
        return cast(
            "PaymentAttempt",
            await self._request_async(
                "get",
                "/v1/orchestration/payment_attempts/{payment_attempt_record}".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.delegated_checkout._order import Order
    from stripe.params.delegated_checkout._requested_session_order_list_params import (
        RequestedSessionOrderListParams,
    )


class RequestedSessionOrderService(StripeService):
    def list(
        self,
        requested_session: str,
        params: Optional["RequestedSessionOrderListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Order]":
        """
        Lists orders for a delegated checkout requested session.
        """
        return cast(
            "ListObject[Order]",
            self._request(
                "get",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/orders".format(
                    requested_session=sanitize_id(requested_session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        requested_session: str,
        params: Optional["RequestedSessionOrderListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Order]":
        """
        Lists orders for a delegated checkout requested session.
        """
        return cast(
            "ListObject[Order]",
            await self._request_async(
                "get",
                "/v1/delegated_checkout/requested_sessions/{requested_session}/orders".format(
                    requested_session=sanitize_id(requested_session),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._payment_location_capability import PaymentLocationCapability
    from stripe._request_options import RequestOptions
    from stripe.params._payment_location_capability_list_params import (
        PaymentLocationCapabilityListParams,
    )
    from stripe.params._payment_location_capability_retrieve_params import (
        PaymentLocationCapabilityRetrieveParams,
    )
    from stripe.params._payment_location_capability_update_params import (
        PaymentLocationCapabilityUpdateParams,
    )


class PaymentLocationCapabilityService(StripeService):
    def list(
        self,
        params: "PaymentLocationCapabilityListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PaymentLocationCapability]":
        """
        List all payment location capabilities associated with the payment location.
        """
        return cast(
            "ListObject[PaymentLocationCapability]",
            self._request(
                "get",
                "/v1/payment_location_capabilities",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "PaymentLocationCapabilityListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PaymentLocationCapability]":
        """
        List all payment location capabilities associated with the payment location.
        """
        return cast(
            "ListObject[PaymentLocationCapability]",
            await self._request_async(
                "get",
                "/v1/payment_location_capabilities",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        capability: str,
        params: "PaymentLocationCapabilityRetrieveParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentLocationCapability":
        """
        Retrieves a payment_location capability
        """
        return cast(
            "PaymentLocationCapability",
            self._request(
                "get",
                "/v1/payment_location_capabilities/{capability}".format(
                    capability=sanitize_id(capability),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        capability: str,
        params: "PaymentLocationCapabilityRetrieveParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentLocationCapability":
        """
        Retrieves a payment_location capability
        """
        return cast(
            "PaymentLocationCapability",
            await self._request_async(
                "get",
                "/v1/payment_location_capabilities/{capability}".format(
                    capability=sanitize_id(capability),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        capability: str,
        params: "PaymentLocationCapabilityUpdateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentLocationCapability":
        """
        Updates a payment_location capability. Request or remove a payment_location capability by updating its requested parameter.
        """
        return cast(
            "PaymentLocationCapability",
            self._request(
                "post",
                "/v1/payment_location_capabilities/{capability}".format(
                    capability=sanitize_id(capability),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        capability: str,
        params: "PaymentLocationCapabilityUpdateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentLocationCapability":
        """
        Updates a payment_location capability. Request or remove a payment_location capability by updating its requested parameter.
        """
        return cast(
            "PaymentLocationCapability",
            await self._request_async(
                "post",
                "/v1/payment_location_capabilities/{capability}".format(
                    capability=sanitize_id(capability),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, Optional
from typing_extensions import Literal


class SettlementAllocationIntentSplit(StripeObject):
    """
    SettlementAllocationIntentSplit resource.
    """

    OBJECT_NAME: ClassVar[
        Literal["v2.payments.settlement_allocation_intent_split"]
    ] = "v2.payments.settlement_allocation_intent_split"

    class Flow(StripeObject):
        outbound_payment: Optional[str]
        """
        If applicable, the ID of the OutboundPayment that created this transaction.
        """
        outbound_transfer: Optional[str]
        """
        If applicable, the ID of the OutboundTransfer that created this transaction.
        """
        received_credit: Optional[str]
        """
        If applicable, the ID of the ReceivedCredit that created this transaction.
        """
        type: Literal[
            "outbound_payment", "outbound_transfer", "received_credit"
        ]
        """
        Type of the flow linked to the transaction which settled the SettlementAllocationIntentSplit. The field matching this value will contain the ID of the flow.
        """

    account: str
    """
    The account id against which the SettlementAllocationIntentSplit should be settled.
    """
    amount: Amount
    """
    The amount and currency of the SettlementAllocationIntentSplit.
    """
    created: str
    """
    Timestamp at which SettlementAllocationIntentSplit was created.
    """
    flow: Optional[Flow]
    """
    Details about the Flow object that settled the split.
    """
    id: str
    """
    Unique identifier for the SettlementAllocationIntentSplit.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Metadata associated with the SettlementAllocationIntentSplit.
    """
    object: Literal["v2.payments.settlement_allocation_intent_split"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    settlement_allocation_intent: str
    """
    The ID of the SettlementAllocationIntent that this split belongs to.
    """
    status: Literal["canceled", "pending", "settled"]
    """
    The status of the SettlementAllocationIntentSplit.
    """
    type: Literal["credit", "debit"]
    """
    The type of the SettlementAllocationIntentSplit.
    """
    _inner_class_types = {"flow": Flow}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.identity._blocklist_entry_create_params import (
        BlocklistEntryCreateParams as BlocklistEntryCreateParams,
    )
    from stripe.params.identity._blocklist_entry_disable_params import (
        BlocklistEntryDisableParams as BlocklistEntryDisableParams,
    )
    from stripe.params.identity._blocklist_entry_list_params import (
        BlocklistEntryListParams as BlocklistEntryListParams,
        BlocklistEntryListParamsCreated as BlocklistEntryListParamsCreated,
    )
    from stripe.params.identity._blocklist_entry_retrieve_params import (
        BlocklistEntryRetrieveParams as BlocklistEntryRetrieveParams,
    )
    from stripe.params.identity._verification_report_list_params import (
        VerificationReportListParams as VerificationReportListParams,
        VerificationReportListParamsCreated as VerificationReportListParamsCreated,
    )
    from stripe.params.identity._verification_report_retrieve_params import (
        VerificationReportRetrieveParams as VerificationReportRetrieveParams,
    )
    from stripe.params.identity._verification_session_cancel_params import (
        VerificationSessionCancelParams as VerificationSessionCancelParams,
    )
    from stripe.params.identity._verification_session_create_params import (
        VerificationSessionCreateParams as VerificationSessionCreateParams,
        VerificationSessionCreateParamsOptions as VerificationSessionCreateParamsOptions,
        VerificationSessionCreateParamsOptionsDocument as VerificationSessionCreateParamsOptionsDocument,
        VerificationSessionCreateParamsProvidedDetails as VerificationSessionCreateParamsProvidedDetails,
        VerificationSessionCreateParamsRelatedPerson as VerificationSessionCreateParamsRelatedPerson,
    )
    from stripe.params.identity._verification_session_list_params import (
        VerificationSessionListParams as VerificationSessionListParams,
        VerificationSessionListParamsCreated as VerificationSessionListParamsCreated,
    )
    from stripe.params.identity._verification_session_modify_params import (
        VerificationSessionModifyParams as VerificationSessionModifyParams,
        VerificationSessionModifyParamsOptions as VerificationSessionModifyParamsOptions,
        VerificationSessionModifyParamsOptionsDocument as VerificationSessionModifyParamsOptionsDocument,
        VerificationSessionModifyParamsProvidedDetails as VerificationSessionModifyParamsProvidedDetails,
    )
    from stripe.params.identity._verification_session_redact_params import (
        VerificationSessionRedactParams as VerificationSessionRedactParams,
    )
    from stripe.params.identity._verification_session_retrieve_params import (
        VerificationSessionRetrieveParams as VerificationSessionRetrieveParams,
    )
    from stripe.params.identity._verification_session_update_params import (
        VerificationSessionUpdateParams as VerificationSessionUpdateParams,
        VerificationSessionUpdateParamsOptions as VerificationSessionUpdateParamsOptions,
        VerificationSessionUpdateParamsOptionsDocument as VerificationSessionUpdateParamsOptionsDocument,
        VerificationSessionUpdateParamsProvidedDetails as VerificationSessionUpdateParamsProvidedDetails,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "BlocklistEntryCreateParams": (
        "stripe.params.identity._blocklist_entry_create_params",
        False,
    ),
    "BlocklistEntryDisableParams": (
        "stripe.params.identity._blocklist_entry_disable_params",
        False,
    ),
    "BlocklistEntryListParams": (
        "stripe.params.identity._blocklist_entry_list_params",
        False,
    ),
    "BlocklistEntryListParamsCreated": (
        "stripe.params.identity._blocklist_entry_list_params",
        False,
    ),
    "BlocklistEntryRetrieveParams": (
        "stripe.params.identity._blocklist_entry_retrieve_params",
        False,
    ),
    "VerificationReportListParams": (
        "stripe.params.identity._verification_report_list_params",
        False,
    ),
    "VerificationReportListParamsCreated": (
        "stripe.params.identity._verification_report_list_params",
        False,
    ),
    "VerificationReportRetrieveParams": (
        "stripe.params.identity._verification_report_retrieve_params",
        False,
    ),
    "VerificationSessionCancelParams": (
        "stripe.params.identity._verification_session_cancel_params",
        False,
    ),
    "VerificationSessionCreateParams": (
        "stripe.params.identity._verification_session_create_params",
        False,
    ),
    "VerificationSessionCreateParamsOptions": (
        "stripe.params.identity._verification_session_create_params",
        False,
    ),
    "VerificationSessionCreateParamsOptionsDocument": (
        "stripe.params.identity._verification_session_create_params",
        False,
    ),
    "VerificationSessionCreateParamsProvidedDetails": (
        "stripe.params.identity._verification_session_create_params",
        False,
    ),
    "VerificationSessionCreateParamsRelatedPerson": (
        "stripe.params.identity._verification_session_create_params",
        False,
    ),
    "VerificationSessionListParams": (
        "stripe.params.identity._verification_session_list_params",
        False,
    ),
    "VerificationSessionListParamsCreated": (
        "stripe.params.identity._verification_session_list_params",
        False,
    ),
    "VerificationSessionModifyParams": (
        "stripe.params.identity._verification_session_modify_params",
        False,
    ),
    "VerificationSessionModifyParamsOptions": (
        "stripe.params.identity._verification_session_modify_params",
        False,
    ),
    "VerificationSessionModifyParamsOptionsDocument": (
        "stripe.params.identity._verification_session_modify_params",
        False,
    ),
    "VerificationSessionModifyParamsProvidedDetails": (
        "stripe.params.identity._verification_session_modify_params",
        False,
    ),
    "VerificationSessionRedactParams": (
        "stripe.params.identity._verification_session_redact_params",
        False,
    ),
    "VerificationSessionRetrieveParams": (
        "stripe.params.identity._verification_session_retrieve_params",
        False,
    ),
    "VerificationSessionUpdateParams": (
        "stripe.params.identity._verification_session_update_params",
        False,
    ),
    "VerificationSessionUpdateParamsOptions": (
        "stripe.params.identity._verification_session_update_params",
        False,
    ),
    "VerificationSessionUpdateParamsOptionsDocument": (
        "stripe.params.identity._verification_session_update_params",
        False,
    ),
    "VerificationSessionUpdateParamsProvidedDetails": (
        "stripe.params.identity._verification_session_update_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._capability import Capability
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._account_capability_list_params import (
        AccountCapabilityListParams,
    )
    from stripe.params._account_capability_retrieve_params import (
        AccountCapabilityRetrieveParams,
    )
    from stripe.params._account_capability_update_params import (
        AccountCapabilityUpdateParams,
    )


class AccountCapabilityService(StripeService):
    def list(
        self,
        account: str,
        params: Optional["AccountCapabilityListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Capability]":
        """
        Returns a list of capabilities associated with the account. The capabilities are returned sorted by creation date, with the most recent capability appearing first.
        """
        return cast(
            "ListObject[Capability]",
            self._request(
                "get",
                "/v1/accounts/{account}/capabilities".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        account: str,
        params: Optional["AccountCapabilityListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Capability]":
        """
        Returns a list of capabilities associated with the account. The capabilities are returned sorted by creation date, with the most recent capability appearing first.
        """
        return cast(
            "ListObject[Capability]",
            await self._request_async(
                "get",
                "/v1/accounts/{account}/capabilities".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        account: str,
        capability: str,
        params: Optional["AccountCapabilityRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Capability":
        """
        Retrieves information about the specified Account Capability.
        """
        return cast(
            "Capability",
            self._request(
                "get",
                "/v1/accounts/{account}/capabilities/{capability}".format(
                    account=sanitize_id(account),
                    capability=sanitize_id(capability),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        account: str,
        capability: str,
        params: Optional["AccountCapabilityRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Capability":
        """
        Retrieves information about the specified Account Capability.
        """
        return cast(
            "Capability",
            await self._request_async(
                "get",
                "/v1/accounts/{account}/capabilities/{capability}".format(
                    account=sanitize_id(account),
                    capability=sanitize_id(capability),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        account: str,
        capability: str,
        params: Optional["AccountCapabilityUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Capability":
        """
        Updates an existing Account Capability. Request or remove a capability by updating its requested parameter.
        """
        return cast(
            "Capability",
            self._request(
                "post",
                "/v1/accounts/{account}/capabilities/{capability}".format(
                    account=sanitize_id(account),
                    capability=sanitize_id(capability),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        account: str,
        capability: str,
        params: Optional["AccountCapabilityUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Capability":
        """
        Updates an existing Account Capability. Request or remove a capability by updating its requested parameter.
        """
        return cast(
            "Capability",
            await self._request_async(
                "post",
                "/v1/accounts/{account}/capabilities/{capability}".format(
                    account=sanitize_id(account),
                    capability=sanitize_id(capability),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from stripe.v2._amount import AmountParam
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class OffSessionPaymentCreateParams(TypedDict):
    amount: AmountParam
    """
    The “presentment amount” to be collected from the customer.
    """
    amount_details: NotRequired["OffSessionPaymentCreateParamsAmountDetails"]
    """
    Provides industry-specific information about the amount.
    """
    application_fee_amount: NotRequired[AmountParam]
    """
    The amount of the application fee (if any) that will be requested to be applied to the
    payment and transferred to the application owner's Stripe account.
    """
    cadence: Literal["recurring", "unscheduled"]
    """
    The frequency of the underlying payment.
    """
    capture: NotRequired["OffSessionPaymentCreateParamsCapture"]
    """
    Details about the capture configuration for the OffSessionPayment.
    """
    customer: str
    """
    ID of the Customer to which this OffSessionPayment belongs.
    """
    description: NotRequired[str]
    """
    An arbitrary string attached to the object. Often useful for displaying to users.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can
    attach to an object. This can be useful for storing additional information about
    the object in a structured format. Learn more about
    [storing information in metadata](https://docs.stripe.com/payments/payment-intents#storing-information-in-metadata).
    """
    on_behalf_of: NotRequired[str]
    """
    The account (if any) for which the funds of the OffSessionPayment are intended.
    """
    payment_details: NotRequired["OffSessionPaymentCreateParamsPaymentDetails"]
    """
    Provides industry-specific information about the payment.
    """
    payment_method: NotRequired[str]
    """
    ID of the payment method used in this OffSessionPayment.
    """
    payment_method_data: NotRequired[
        "OffSessionPaymentCreateParamsPaymentMethodData"
    ]
    """
    If provided, this hash will be used to create a PaymentMethod. The new PaymentMethod will appear in the payment_method property on the OffSessionPayment.
    """
    payment_method_options: NotRequired[
        "OffSessionPaymentCreateParamsPaymentMethodOptions"
    ]
    """
    Payment method options for the off-session payment.
    """
    payments_orchestration: NotRequired[
        "OffSessionPaymentCreateParamsPaymentsOrchestration"
    ]
    """
    Details about the payments orchestration configuration.
    """
    retry_details: NotRequired["OffSessionPaymentCreateParamsRetryDetails"]
    """
    Details about the OffSessionPayment retries.
    """
    statement_descriptor: NotRequired[str]
    """
    Text that appears on the customer's statement as the statement descriptor for a
    non-card charge. This value overrides the account's default statement descriptor.
    For information about requirements, including the 22-character limit, see the
    [Statement Descriptor docs](https://docs.stripe.com/get-started/account/statement-descriptors).
    """
    statement_descriptor_suffix: NotRequired[str]
    """
    Provides information about a card charge. Concatenated to the account's
    [statement descriptor prefix](https://docs.stripe.com/get-started/account/statement-descriptors#static)
    to form the complete statement descriptor that appears on the customer's statement.
    """
    test_clock: NotRequired[str]
    """
    Test clock that can be used to advance the retry attempts in a sandbox.
    """
    transfer_data: NotRequired["OffSessionPaymentCreateParamsTransferData"]
    """
    The data that automatically creates a Transfer after the payment finalizes. Learn more about the use case for [connected accounts](https://docs.stripe.com/payments/connected-accounts).
    """


class OffSessionPaymentCreateParamsAmountDetails(TypedDict):
    discount_amount: NotRequired[int]
    """
    The amount the total transaction was discounted for.
    """
    enforce_arithmetic_validation: NotRequired[bool]
    """
    Set to `false` to return arithmetic validation errors in the response without failing the request. Use this when you want the operation to proceed regardless of arithmetic errors in the line item data.
    Omit or set to `true` to immediately return a 400 error when arithmetic validation fails. Use this for strict validation that prevents processing with line item data that has arithmetic inconsistencies.
    For card payments, Stripe doesn't send line item data to card networks if there's an arithmetic validation error.
    """
    line_items: NotRequired[
        List["OffSessionPaymentCreateParamsAmountDetailsLineItem"]
    ]
    """
    A list of line items, each containing information about a product in the OffSessionPayment. There is a maximum of 10 line items.
    """
    shipping: NotRequired["OffSessionPaymentCreateParamsAmountDetailsShipping"]
    """
    Contains information about the shipping portion of the amount.
    """
    tax: NotRequired["OffSessionPaymentCreateParamsAmountDetailsTax"]
    """
    Contains information about the tax portion of the amount.
    """


class OffSessionPaymentCreateParamsAmountDetailsLineItem(TypedDict):
    discount_amount: NotRequired[int]
    """
    The amount an item was discounted for. Positive integer.
    """
    product_code: NotRequired[str]
    """
    Unique identifier of the product. At most 12 characters long.
    """
    product_name: str
    """
    Name of the product. At most 100 characters long.
    """
    quantity: int
    """
    Number of items of the product. Positive integer.
    """
    tax: NotRequired["OffSessionPaymentCreateParamsAmountDetailsLineItemTax"]
    """
    Contains information about the tax on the item.
    """
    unit_cost: int
    """
    Cost of the product. Positive integer.
    """
    unit_of_measure: NotRequired[str]
    """
    A unit of measure for the line item, such as gallons, feet, meters, etc.
    The maximum length is 12 characters.
    """


class OffSessionPaymentCreateParamsAmountDetailsLineItemTax(TypedDict):
    total_tax_amount: int
    """
    Total portion of the amount that is for tax.
    """


class OffSessionPaymentCreateParamsAmountDetailsShipping(TypedDict):
    amount: NotRequired[int]
    """
    Portion of the amount that is for shipping.
    """
    from_postal_code: NotRequired[str]
    """
    The postal code that represents the shipping source.
    """
    to_postal_code: NotRequired[str]
    """
    The postal code that represents the shipping destination.
    """


class OffSessionPaymentCreateParamsAmountDetailsTax(TypedDict):
    total_tax_amount: int
    """
    Total portion of the amount that is for tax.
    """


class OffSessionPaymentCreateParamsCapture(TypedDict):
    capture_method: Literal["automatic", "manual"]
    """
    The method to use to capture the payment.
    """


class OffSessionPaymentCreateParamsPaymentDetails(TypedDict):
    customer_reference: NotRequired[str]
    """
    A unique value to identify the customer. This field is applicable only for card payments. For card payments, this field is truncated to 25 alphanumeric characters, excluding spaces, before being sent to card networks.
    """
    order_reference: NotRequired[str]
    """
    A unique value assigned by the business to identify the transaction. Required for L2 and L3 rates.
    For Cards, this field is truncated to 25 alphanumeric characters, excluding spaces, before being sent to card networks.
    """


class OffSessionPaymentCreateParamsPaymentMethodData(TypedDict):
    billing_details: NotRequired[
        "OffSessionPaymentCreateParamsPaymentMethodDataBillingDetails"
    ]
    """
    Billing information associated with the payment method.
    """
    card: NotRequired["OffSessionPaymentCreateParamsPaymentMethodDataCard"]
    """
    Contains card details that can be used to create a card PaymentMethod for PCI compliant users.
    """
    type: Literal["card"]
    """
    The type of the PaymentMethod. An additional hash is included on the PaymentMethod with a name matching this value. It contains additional information specific to the PaymentMethod type.
    """


class OffSessionPaymentCreateParamsPaymentMethodDataBillingDetails(TypedDict):
    address: NotRequired[
        "OffSessionPaymentCreateParamsPaymentMethodDataBillingDetailsAddress"
    ]
    """
    Billing address.
    """
    email: NotRequired[str]
    """
    Email address.
    """
    name: NotRequired[str]
    """
    Full name.
    """
    phone: NotRequired[str]
    """
    Billing phone number (including extension).
    """


class OffSessionPaymentCreateParamsPaymentMethodDataBillingDetailsAddress(
    TypedDict,
):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code (ISO 3166-1 alpha-2).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region (ISO 3166-2).
    """


class OffSessionPaymentCreateParamsPaymentMethodDataCard(TypedDict):
    cvc: NotRequired[str]
    """
    The card CVC.
    """
    exp_month: str
    """
    The card expiration month.
    """
    exp_year: str
    """
    The card expiration year.
    """
    number: NotRequired[str]
    """
    The card number.
    """


class OffSessionPaymentCreateParamsPaymentMethodOptions(TypedDict):
    card: NotRequired["OffSessionPaymentCreateParamsPaymentMethodOptionsCard"]
    """
    Payment method options for the card payment type.
    """


class OffSessionPaymentCreateParamsPaymentMethodOptionsCard(TypedDict):
    mcc: NotRequired[str]
    """
    The merchant category code for this transaction. Used in interchange and authorization to improve auth rates.
    """
    network_transaction_id: NotRequired[str]
    """
    If you are making a Credential On File transaction with a previously saved card, you should pass the Network Transaction ID
    from a prior initial authorization on Stripe (from a successful SetupIntent or a PaymentIntent with `setup_future_usage` set),
    or one that you have obtained from another payment processor. This is a token from the network which uniquely identifies the transaction.
    Visa calls this the Transaction ID, Mastercard calls this the Trace ID, and American Express calls this the Acquirer Reference Data.
    Note that you should pass in a Network Transaction ID if you have one, regardless of whether this is a
    Customer-Initiated Transaction (CIT) or a Merchant-Initiated Transaction (MIT).
    """


class OffSessionPaymentCreateParamsPaymentsOrchestration(TypedDict):
    enabled: bool
    """
    True when you want to enable payments orchestration for this off-session payment. False otherwise.
    """


class OffSessionPaymentCreateParamsRetryDetails(TypedDict):
    retry_policy: NotRequired[str]
    """
    The pre-configured retry policy to use for the payment.
    """
    retry_strategy: NotRequired[Literal["best_available", "none"]]
    """
    Indicates the strategy for how you want Stripe to retry the payment.
    """


class OffSessionPaymentCreateParamsTransferData(TypedDict):
    amount: NotRequired[int]
    """
    The amount transferred to the destination account. This transfer will occur
    automatically after the payment succeeds. If no amount is specified, by default
    the entire payment amount is transferred to the destination account. The amount
    must be less than or equal to the
    [amount_requested](https://docs.stripe.com/api/v2/off-session-payments/object?api-version=2025-05-28.preview#v2_off_session_payment_object-amount_requested),
    and must be a positive integer representing how much to transfer in the smallest
    currency unit (e.g., 100 cents to charge $1.00).
    """
    destination: str
    """
    The account (if any) that the payment is attributed to for tax reporting, and
    where funds from the payment are transferred to after payment success.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class FinancialAddress(StripeObject):
    """
    A Financial Address contains information needed to transfer money to a Financial Account. A Financial Account can have more than one Financial Address.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.financial_address"]] = (
        "v2.money_management.financial_address"
    )

    class Credentials(StripeObject):
        class CaBankAccount(StripeObject):
            account_holder_name: str
            """
            The account holder name to be used during bank transfers.
            """
            account_number: Optional[str]
            """
            The account number of the Canadian Bank Account.
            """
            bank_name: str
            """
            The name of the Bank.
            """
            institution_number: str
            """
            The institution number of the Canadian Bank Account.
            """
            last4: str
            """
            The last four digits of the Canadian Bank Account number. This will always be returned.
            To view the full account number when retrieving or listing FinancialAddresses, use the `include` request parameter.
            """
            transit_number: str
            """
            The transit number of the Canadian Bank Account.
            """

        class CryptoWallet(StripeObject):
            address: str
            """
            The blockchain address of the crypto wallet.
            """
            memo: Optional[str]
            """
            Required if the network supports memos (e.g. Stellar).
            """
            network: Literal[
                "arbitrum",
                "avalanche_c_chain",
                "base",
                "ethereum",
                "optimism",
                "polygon",
                "solana",
                "stellar",
                "tempo",
            ]
            """
            The blockchain network of the crypto wallet.
            """

        class GbBankAccount(StripeObject):
            account_holder_name: str
            """
            The account holder name to be used during bank transference.
            """
            account_number: Optional[str]
            """
            The account number of the UK Bank Account.
            """
            last4: str
            """
            The last four digits of the UK Bank Account number. This will always be returned.
            To view the full account number when retrieving or listing FinancialAddresses, use the `include` request parameter.
            """
            sort_code: str
            """
            The sort code of the UK Bank Account.
            """

        class MxBankAccount(StripeObject):
            account_holder_name: str
            """
            The account holder name to be used during bank transfers.
            """
            clabe: str
            """
            The CLABE (Clave Bancaria Estandarizada) of the Mexican Bank Account.
            """

        class SepaBankAccount(StripeObject):
            account_holder_name: str
            """
            The account holder name to be used during bank transfers.
            """
            bank_name: str
            """
            The name of the Bank.
            """
            bic: str
            """
            The BIC of the SEPA Bank Account.
            """
            country: str
            """
            The originating country of the SEPA Bank account.
            """
            iban: str
            """
            The IBAN of the SEPA Bank Account.
            """
            last4: str
            """
            The last four digits of the SEPA Bank Account number. This will always be returned.
            To view the full account number when retrieving or listing FinancialAddresses, use the `include` request parameter.
            """

        class UsBankAccount(StripeObject):
            class AccountHolderAddress(StripeObject):
                city: str
                """
                The city of the address.
                """
                country: str
                """
                The country of the address.
                """
                line1: str
                """
                The first line of the address.
                """
                line2: str
                """
                The second line of the address.
                """
                postal_code: str
                """
                The postal / zip code of the address.
                """
                state: str
                """
                The state of the address.
                """
                town: str
                """
                The town of the address.
                """

            account_holder_address: Optional[AccountHolderAddress]
            """
            The address of the account holder.
            """
            account_holder_name: Optional[str]
            """
            The name of the account holder.
            """
            account_number: Optional[str]
            """
            The account number of the US Bank Account.
            """
            bank_name: Optional[str]
            """
            The name of the Bank.
            """
            bic: Optional[str]
            """
            The BIC of the bank or financial institution.
            """
            last4: str
            """
            The last four digits of the US Bank Account number. This will always be returned.
            To view the full account number when retrieving or listing FinancialAddresses, use the `include` request parameter.
            """
            routing_number: str
            """
            The routing number of the US Bank Account.
            """
            _inner_class_types = {
                "account_holder_address": AccountHolderAddress,
            }

        ca_bank_account: Optional[CaBankAccount]
        """
        The credentials of the Canadian Bank Account for the FinancialAddress. This contains unique banking details such as the account number, institution number, etc. of a Canadian bank account.
        """
        crypto_wallet: Optional[CryptoWallet]
        """
        The credentials of the crypto wallet for the Financial Address. This contains unique details such as the blockchain network, wallet address, and memo of a crypto wallet.
        """
        gb_bank_account: Optional[GbBankAccount]
        """
        The credentials of the UK Bank Account for the FinancialAddress. This contains unique banking details such as the sort code, account number, etc. of a UK bank account.
        """
        mx_bank_account: Optional[MxBankAccount]
        """
        The credentials of the Mexican Bank Account for the FinancialAddress. This contains unique banking details such as the CLABE and account holder name of a Mexican bank account.
        """
        sepa_bank_account: Optional[SepaBankAccount]
        """
        The credentials of the SEPA Bank Account for the FinancialAddress. This contains unique banking details such as the IBAN, BIC, etc. of a SEPA bank account.
        """
        type: Literal[
            "ca_bank_account",
            "crypto_wallet",
            "gb_bank_account",
            "mx_bank_account",
            "sepa_bank_account",
            "us_bank_account",
        ]
        """
        Open Enum. The type of Credentials that are provisioned for the FinancialAddress.
        """
        us_bank_account: Optional[UsBankAccount]
        """
        The credentials of the US Bank Account for the FinancialAddress. This contains unique banking details such as the routing number, account number, etc. of a US bank account.
        """
        _inner_class_types = {
            "ca_bank_account": CaBankAccount,
            "crypto_wallet": CryptoWallet,
            "gb_bank_account": GbBankAccount,
            "mx_bank_account": MxBankAccount,
            "sepa_bank_account": SepaBankAccount,
            "us_bank_account": UsBankAccount,
        }

    created: str
    """
    The creation timestamp of the FinancialAddress.
    """
    credentials: Optional[Credentials]
    """
    Object indicates the type of credentials that have been allocated and attached to the FinancialAddress.
    It contains all necessary banking details with which to perform money movements with the FinancialAddress.
    This field is only available for FinancialAddresses with an active status.
    """
    currency: str
    """
    Open Enum. The currency the FinancialAddress supports.
    """
    financial_account: str
    """
    A ID of the FinancialAccount this FinancialAddress corresponds to.
    """
    id: str
    """
    The ID of a FinancialAddress.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.money_management.financial_address"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    settlement_currency: Optional[str]
    """
    Open Enum. The currency the FinancialAddress settles into the FinancialAccount.
    """
    status: Literal["active", "archived", "failed", "pending"]
    """
    Closed Enum. An enum representing the status of the FinancialAddress. This indicates whether or not the FinancialAddress can be used for any money movement flows.
    """
    _inner_class_types = {"credentials": Credentials}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
import json
from stripe._api_version import _ApiVersion
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from uuid import uuid4
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._cash_balance import CashBalance
    from stripe._request_options import RequestOptions
    from stripe.params._customer_cash_balance_retrieve_params import (
        CustomerCashBalanceRetrieveParams,
    )
    from stripe.params._customer_cash_balance_update_params import (
        CustomerCashBalanceUpdateParams,
    )


class CustomerCashBalanceService(StripeService):
    def retrieve(
        self,
        customer: str,
        params: Optional["CustomerCashBalanceRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CashBalance":
        """
        Retrieves a customer's cash balance.
        """
        return cast(
            "CashBalance",
            self._request(
                "get",
                "/v1/customers/{customer}/cash_balance".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        customer: str,
        params: Optional["CustomerCashBalanceRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CashBalance":
        """
        Retrieves a customer's cash balance.
        """
        return cast(
            "CashBalance",
            await self._request_async(
                "get",
                "/v1/customers/{customer}/cash_balance".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        customer: str,
        params: Optional["CustomerCashBalanceUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CashBalance":
        """
        Changes the settings on a customer's cash balance.
        """
        return cast(
            "CashBalance",
            self._request(
                "post",
                "/v1/customers/{customer}/cash_balance".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        customer: str,
        params: Optional["CustomerCashBalanceUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CashBalance":
        """
        Changes the settings on a customer's cash balance.
        """
        return cast(
            "CashBalance",
            await self._request_async(
                "post",
                "/v1/customers/{customer}/cash_balance".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def serialize_batch_update(
        self,
        customer: str,
        params: Optional["CustomerCashBalanceUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a CustomerCashBalance update request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {"customer": customer},
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class AccountEvaluation(StripeObject):
    """
    Account Evaluation resource.
    """

    OBJECT_NAME: ClassVar[Literal["v2.core.account_evaluation"]] = (
        "v2.core.account_evaluation"
    )

    class AccountData(StripeObject):
        class Defaults(StripeObject):
            class Profile(StripeObject):
                business_url: str
                """
                The business URL.
                """
                doing_business_as: Optional[str]
                """
                Doing business as (DBA) name.
                """
                product_description: Optional[str]
                """
                Description of the account's product or service.
                """

            profile: Profile
            """
            Account profile data.
            """
            _inner_class_types = {"profile": Profile}

        class Identity(StripeObject):
            class BusinessDetails(StripeObject):
                registered_name: Optional[str]
                """
                Registered business name.
                """

            business_details: BusinessDetails
            """
            Business details for identity data.
            """
            _inner_class_types = {"business_details": BusinessDetails}

        defaults: Optional[Defaults]
        """
        Default account settings.
        """
        identity: Optional[Identity]
        """
        Identity data.
        """
        _inner_class_types = {"defaults": Defaults, "identity": Identity}

    account: Optional[str]
    """
    The account ID if this evaluation is for an existing account.
    """
    account_data: Optional[AccountData]
    """
    Account data if this evaluation is for an account without an existing Stripe entity.
    """
    created: str
    """
    Timestamp at which the evaluation was created.
    """
    evaluations_triggered: List[Literal["fraudulent_website"]]
    """
    List of signals that were triggered for evaluation.
    """
    id: str
    """
    Unique identifier for the account evaluation.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.core.account_evaluation"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    _inner_class_types = {"account_data": AccountData}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired


class DisputeProvisionalCreditParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    grant_deadline: NotRequired[int]
    """
    Override the deadline by which the platform must grant a provisional credit to the consumer.
    """
    revocable_after: NotRequired[int]
    """
    Override the earliest time after which the platform may revoke the provisional credit.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._account_link_service import AccountLinkService
    from stripe._account_notice_service import AccountNoticeService
    from stripe._account_service import AccountService
    from stripe._account_session_service import AccountSessionService
    from stripe._apple_pay_domain_service import ApplePayDomainService
    from stripe._application_fee_service import ApplicationFeeService
    from stripe._apps_service import AppsService
    from stripe._balance_service import BalanceService
    from stripe._balance_settings_service import BalanceSettingsService
    from stripe._balance_transaction_service import BalanceTransactionService
    from stripe._balance_transfer_service import BalanceTransferService
    from stripe._billing_portal_service import BillingPortalService
    from stripe._billing_service import BillingService
    from stripe._capital_service import CapitalService
    from stripe._charge_service import ChargeService
    from stripe._checkout_service import CheckoutService
    from stripe._climate_service import ClimateService
    from stripe._confirmation_token_service import ConfirmationTokenService
    from stripe._country_spec_service import CountrySpecService
    from stripe._coupon_service import CouponService
    from stripe._credit_note_service import CreditNoteService
    from stripe._crypto_service import CryptoService
    from stripe._customer_service import CustomerService
    from stripe._customer_session_service import CustomerSessionService
    from stripe._delegated_checkout_service import DelegatedCheckoutService
    from stripe._dispute_service import DisputeService
    from stripe._entitlements_service import EntitlementsService
    from stripe._ephemeral_key_service import EphemeralKeyService
    from stripe._event_service import EventService
    from stripe._exchange_rate_service import ExchangeRateService
    from stripe._external_account_service import ExternalAccountService
    from stripe._file_link_service import FileLinkService
    from stripe._file_service import FileService
    from stripe._financial_connections_service import (
        FinancialConnectionsService,
    )
    from stripe._forwarding_service import ForwardingService
    from stripe._fr_meal_vouchers_onboarding_service import (
        FrMealVouchersOnboardingService,
    )
    from stripe._fx_quote_service import FxQuoteService
    from stripe._gift_card_operation_service import GiftCardOperationService
    from stripe._gift_card_service import GiftCardService
    from stripe._identity_service import IdentityService
    from stripe._invoice_item_service import InvoiceItemService
    from stripe._invoice_payment_service import InvoicePaymentService
    from stripe._invoice_rendering_template_service import (
        InvoiceRenderingTemplateService,
    )
    from stripe._invoice_service import InvoiceService
    from stripe._issuing_service import IssuingService
    from stripe._mandate_service import MandateService
    from stripe._margin_service import MarginService
    from stripe._orchestration_service import OrchestrationService
    from stripe._order_service import OrderService
    from stripe._payment_attempt_record_service import (
        PaymentAttemptRecordService,
    )
    from stripe._payment_intent_service import PaymentIntentService
    from stripe._payment_link_service import PaymentLinkService
    from stripe._payment_location_capability_service import (
        PaymentLocationCapabilityService,
    )
    from stripe._payment_location_service import PaymentLocationService
    from stripe._payment_method_configuration_service import (
        PaymentMethodConfigurationService,
    )
    from stripe._payment_method_domain_service import (
        PaymentMethodDomainService,
    )
    from stripe._payment_method_service import PaymentMethodService
    from stripe._payment_record_service import PaymentRecordService
    from stripe._payout_service import PayoutService
    from stripe._plan_service import PlanService
    from stripe._price_service import PriceService
    from stripe._privacy_service import PrivacyService
    from stripe._product_catalog_service import ProductCatalogService
    from stripe._product_service import ProductService
    from stripe._promotion_code_service import PromotionCodeService
    from stripe._quote_service import QuoteService
    from stripe._radar_service import RadarService
    from stripe._refund_service import RefundService
    from stripe._reporting_service import ReportingService
    from stripe._reserve_service import ReserveService
    from stripe._review_service import ReviewService
    from stripe._setup_attempt_service import SetupAttemptService
    from stripe._setup_intent_service import SetupIntentService
    from stripe._shared_payment_service import SharedPaymentService
    from stripe._shipping_rate_service import ShippingRateService
    from stripe._sigma_service import SigmaService
    from stripe._source_service import SourceService
    from stripe._subscription_item_service import SubscriptionItemService
    from stripe._subscription_schedule_service import (
        SubscriptionScheduleService,
    )
    from stripe._subscription_service import SubscriptionService
    from stripe._tax_code_service import TaxCodeService
    from stripe._tax_fund_service import TaxFundService
    from stripe._tax_id_service import TaxIdService
    from stripe._tax_rate_service import TaxRateService
    from stripe._tax_service import TaxService
    from stripe._terminal_service import TerminalService
    from stripe._test_helpers_service import TestHelpersService
    from stripe._token_service import TokenService
    from stripe._topup_service import TopupService
    from stripe._transfer_service import TransferService
    from stripe._treasury_service import TreasuryService
    from stripe._webhook_endpoint_service import WebhookEndpointService

_subservices = {
    "accounts": ["stripe._account_service", "AccountService"],
    "account_links": ["stripe._account_link_service", "AccountLinkService"],
    "account_notices": [
        "stripe._account_notice_service",
        "AccountNoticeService",
    ],
    "account_sessions": [
        "stripe._account_session_service",
        "AccountSessionService",
    ],
    "apple_pay_domains": [
        "stripe._apple_pay_domain_service",
        "ApplePayDomainService",
    ],
    "application_fees": [
        "stripe._application_fee_service",
        "ApplicationFeeService",
    ],
    "apps": ["stripe._apps_service", "AppsService"],
    "balance": ["stripe._balance_service", "BalanceService"],
    "balance_settings": [
        "stripe._balance_settings_service",
        "BalanceSettingsService",
    ],
    "balance_transactions": [
        "stripe._balance_transaction_service",
        "BalanceTransactionService",
    ],
    "balance_transfers": [
        "stripe._balance_transfer_service",
        "BalanceTransferService",
    ],
    "billing": ["stripe._billing_service", "BillingService"],
    "billing_portal": [
        "stripe._billing_portal_service",
        "BillingPortalService",
    ],
    "capital": ["stripe._capital_service", "CapitalService"],
    "charges": ["stripe._charge_service", "ChargeService"],
    "checkout": ["stripe._checkout_service", "CheckoutService"],
    "climate": ["stripe._climate_service", "ClimateService"],
    "confirmation_tokens": [
        "stripe._confirmation_token_service",
        "ConfirmationTokenService",
    ],
    "country_specs": ["stripe._country_spec_service", "CountrySpecService"],
    "coupons": ["stripe._coupon_service", "CouponService"],
    "credit_notes": ["stripe._credit_note_service", "CreditNoteService"],
    "crypto": ["stripe._crypto_service", "CryptoService"],
    "customers": ["stripe._customer_service", "CustomerService"],
    "customer_sessions": [
        "stripe._customer_session_service",
        "CustomerSessionService",
    ],
    "delegated_checkout": [
        "stripe._delegated_checkout_service",
        "DelegatedCheckoutService",
    ],
    "disputes": ["stripe._dispute_service", "DisputeService"],
    "entitlements": ["stripe._entitlements_service", "EntitlementsService"],
    "ephemeral_keys": ["stripe._ephemeral_key_service", "EphemeralKeyService"],
    "events": ["stripe._event_service", "EventService"],
    "exchange_rates": ["stripe._exchange_rate_service", "ExchangeRateService"],
    "external_accounts": [
        "stripe._external_account_service",
        "ExternalAccountService",
    ],
    "files": ["stripe._file_service", "FileService"],
    "file_links": ["stripe._file_link_service", "FileLinkService"],
    "financial_connections": [
        "stripe._financial_connections_service",
        "FinancialConnectionsService",
    ],
    "forwarding": ["stripe._forwarding_service", "ForwardingService"],
    "fr_meal_vouchers_onboardings": [
        "stripe._fr_meal_vouchers_onboarding_service",
        "FrMealVouchersOnboardingService",
    ],
    "fx_quotes": ["stripe._fx_quote_service", "FxQuoteService"],
    "gift_cards": ["stripe._gift_card_service", "GiftCardService"],
    "gift_card_operations": [
        "stripe._gift_card_operation_service",
        "GiftCardOperationService",
    ],
    "identity": ["stripe._identity_service", "IdentityService"],
    "invoices": ["stripe._invoice_service", "InvoiceService"],
    "invoice_items": ["stripe._invoice_item_service", "InvoiceItemService"],
    "invoice_payments": [
        "stripe._invoice_payment_service",
        "InvoicePaymentService",
    ],
    "invoice_rendering_templates": [
        "stripe._invoice_rendering_template_service",
        "InvoiceRenderingTemplateService",
    ],
    "issuing": ["stripe._issuing_service", "IssuingService"],
    "mandates": ["stripe._mandate_service", "MandateService"],
    "margins": ["stripe._margin_service", "MarginService"],
    "orchestration": ["stripe._orchestration_service", "OrchestrationService"],
    "orders": ["stripe._order_service", "OrderService"],
    "payment_attempt_records": [
        "stripe._payment_attempt_record_service",
        "PaymentAttemptRecordService",
    ],
    "payment_intents": [
        "stripe._payment_intent_service",
        "PaymentIntentService",
    ],
    "payment_links": ["stripe._payment_link_service", "PaymentLinkService"],
    "payment_locations": [
        "stripe._payment_location_service",
        "PaymentLocationService",
    ],
    "payment_location_capabilities": [
        "stripe._payment_location_capability_service",
        "PaymentLocationCapabilityService",
    ],
    "payment_methods": [
        "stripe._payment_method_service",
        "PaymentMethodService",
    ],
    "payment_method_configurations": [
        "stripe._payment_method_configuration_service",
        "PaymentMethodConfigurationService",
    ],
    "payment_method_domains": [
        "stripe._payment_method_domain_service",
        "PaymentMethodDomainService",
    ],
    "payment_records": [
        "stripe._payment_record_service",
        "PaymentRecordService",
    ],
    "payouts": ["stripe._payout_service", "PayoutService"],
    "plans": ["stripe._plan_service", "PlanService"],
    "prices": ["stripe._price_service", "PriceService"],
    "privacy": ["stripe._privacy_service", "PrivacyService"],
    "products": ["stripe._product_service", "ProductService"],
    "product_catalog": [
        "stripe._product_catalog_service",
        "ProductCatalogService",
    ],
    "promotion_codes": [
        "stripe._promotion_code_service",
        "PromotionCodeService",
    ],
    "quotes": ["stripe._quote_service", "QuoteService"],
    "radar": ["stripe._radar_service", "RadarService"],
    "refunds": ["stripe._refund_service", "RefundService"],
    "reporting": ["stripe._reporting_service", "ReportingService"],
    "reserve": ["stripe._reserve_service", "ReserveService"],
    "reviews": ["stripe._review_service", "ReviewService"],
    "setup_attempts": ["stripe._setup_attempt_service", "SetupAttemptService"],
    "setup_intents": ["stripe._setup_intent_service", "SetupIntentService"],
    "shared_payment": [
        "stripe._shared_payment_service",
        "SharedPaymentService",
    ],
    "shipping_rates": ["stripe._shipping_rate_service", "ShippingRateService"],
    "sigma": ["stripe._sigma_service", "SigmaService"],
    "sources": ["stripe._source_service", "SourceService"],
    "subscriptions": ["stripe._subscription_service", "SubscriptionService"],
    "subscription_items": [
        "stripe._subscription_item_service",
        "SubscriptionItemService",
    ],
    "subscription_schedules": [
        "stripe._subscription_schedule_service",
        "SubscriptionScheduleService",
    ],
    "tax": ["stripe._tax_service", "TaxService"],
    "tax_codes": ["stripe._tax_code_service", "TaxCodeService"],
    "tax_funds": ["stripe._tax_fund_service", "TaxFundService"],
    "tax_ids": ["stripe._tax_id_service", "TaxIdService"],
    "tax_rates": ["stripe._tax_rate_service", "TaxRateService"],
    "terminal": ["stripe._terminal_service", "TerminalService"],
    "test_helpers": ["stripe._test_helpers_service", "TestHelpersService"],
    "tokens": ["stripe._token_service", "TokenService"],
    "topups": ["stripe._topup_service", "TopupService"],
    "transfers": ["stripe._transfer_service", "TransferService"],
    "treasury": ["stripe._treasury_service", "TreasuryService"],
    "webhook_endpoints": [
        "stripe._webhook_endpoint_service",
        "WebhookEndpointService",
    ],
}


class V1Services(StripeService):
    accounts: "AccountService"
    account_links: "AccountLinkService"
    account_notices: "AccountNoticeService"
    account_sessions: "AccountSessionService"
    apple_pay_domains: "ApplePayDomainService"
    application_fees: "ApplicationFeeService"
    apps: "AppsService"
    balance: "BalanceService"
    balance_settings: "BalanceSettingsService"
    balance_transactions: "BalanceTransactionService"
    balance_transfers: "BalanceTransferService"
    billing: "BillingService"
    billing_portal: "BillingPortalService"
    capital: "CapitalService"
    charges: "ChargeService"
    checkout: "CheckoutService"
    climate: "ClimateService"
    confirmation_tokens: "ConfirmationTokenService"
    country_specs: "CountrySpecService"
    coupons: "CouponService"
    credit_notes: "CreditNoteService"
    crypto: "CryptoService"
    customers: "CustomerService"
    customer_sessions: "CustomerSessionService"
    delegated_checkout: "DelegatedCheckoutService"
    disputes: "DisputeService"
    entitlements: "EntitlementsService"
    ephemeral_keys: "EphemeralKeyService"
    events: "EventService"
    exchange_rates: "ExchangeRateService"
    external_accounts: "ExternalAccountService"
    files: "FileService"
    file_links: "FileLinkService"
    financial_connections: "FinancialConnectionsService"
    forwarding: "ForwardingService"
    fr_meal_vouchers_onboardings: "FrMealVouchersOnboardingService"
    fx_quotes: "FxQuoteService"
    gift_cards: "GiftCardService"
    gift_card_operations: "GiftCardOperationService"
    identity: "IdentityService"
    invoices: "InvoiceService"
    invoice_items: "InvoiceItemService"
    invoice_payments: "InvoicePaymentService"
    invoice_rendering_templates: "InvoiceRenderingTemplateService"
    issuing: "IssuingService"
    mandates: "MandateService"
    margins: "MarginService"
    orchestration: "OrchestrationService"
    orders: "OrderService"
    payment_attempt_records: "PaymentAttemptRecordService"
    payment_intents: "PaymentIntentService"
    payment_links: "PaymentLinkService"
    payment_locations: "PaymentLocationService"
    payment_location_capabilities: "PaymentLocationCapabilityService"
    payment_methods: "PaymentMethodService"
    payment_method_configurations: "PaymentMethodConfigurationService"
    payment_method_domains: "PaymentMethodDomainService"
    payment_records: "PaymentRecordService"
    payouts: "PayoutService"
    plans: "PlanService"
    prices: "PriceService"
    privacy: "PrivacyService"
    products: "ProductService"
    product_catalog: "ProductCatalogService"
    promotion_codes: "PromotionCodeService"
    quotes: "QuoteService"
    radar: "RadarService"
    refunds: "RefundService"
    reporting: "ReportingService"
    reserve: "ReserveService"
    reviews: "ReviewService"
    setup_attempts: "SetupAttemptService"
    setup_intents: "SetupIntentService"
    shared_payment: "SharedPaymentService"
    shipping_rates: "ShippingRateService"
    sigma: "SigmaService"
    sources: "SourceService"
    subscriptions: "SubscriptionService"
    subscription_items: "SubscriptionItemService"
    subscription_schedules: "SubscriptionScheduleService"
    tax: "TaxService"
    tax_codes: "TaxCodeService"
    tax_funds: "TaxFundService"
    tax_ids: "TaxIdService"
    tax_rates: "TaxRateService"
    terminal: "TerminalService"
    test_helpers: "TestHelpersService"
    tokens: "TokenService"
    topups: "TopupService"
    transfers: "TransferService"
    treasury: "TreasuryService"
    webhook_endpoints: "WebhookEndpointService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class Contract(StripeObject):
    """
    Main Contract resource representing a comprehensive billing agreement
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.contract"]] = (
        "v2.billing.contract"
    )

    class BillingCycleAnchor(StripeObject):
        timestamp: str
        """
        The billing cycle anchor as a UTC timestamp.
        """

    class BillingSettings(StripeObject):
        class BillSettingsDetails(StripeObject):
            class Calculation(StripeObject):
                class Tax(StripeObject):
                    type: Literal["automatic", "manual"]
                    """
                    The type of tax calculation.
                    """

                tax: Optional[Tax]
                """
                Tax calculation settings.
                """
                _inner_class_types = {"tax": Tax}

            class Invoice(StripeObject):
                class TimeUntilDue(StripeObject):
                    interval: Literal["day", "month", "week", "year"]
                    """
                    The interval unit.
                    """
                    interval_count: int
                    """
                    The number of intervals.
                    """

                time_until_due: Optional[TimeUntilDue]
                """
                The number of time units before the invoice is past due.
                """
                _inner_class_types = {"time_until_due": TimeUntilDue}

            calculation: Optional[Calculation]
            """
            Calculation settings.
            """
            invoice: Optional[Invoice]
            """
            Invoice settings.
            """
            _inner_class_types = {
                "calculation": Calculation,
                "invoice": Invoice,
            }

        class BillingProfileDetails(StripeObject):
            customer: str
            """
            The customer who pays for the contract invoice.
            """
            default_payment_method: Optional[str]
            """
            The default payment method for the contract.
            """

        class CollectionSettingsDetails(StripeObject):
            collection_method: Literal["charge_automatically", "send_invoice"]
            """
            The collection method.
            """
            payment_method_configuration: Optional[str]
            """
            The payment method configuration.
            """

        bill_settings_details: Optional[BillSettingsDetails]
        """
        The bill settings details configures invoice and tax settings for the contract.
        """
        billing_profile_details: BillingProfileDetails
        """
        The billing profile details configures who is charged for the contract.
        """
        collection_settings_details: CollectionSettingsDetails
        """
        The collection settings details configures how payments are collected on the contract.
        """
        _inner_class_types = {
            "bill_settings_details": BillSettingsDetails,
            "billing_profile_details": BillingProfileDetails,
            "collection_settings_details": CollectionSettingsDetails,
        }

    class OneTimeFees(StripeObject):
        class Data(StripeObject):
            class BillAt(StripeObject):
                timestamp: str
                """
                The timestamp at which the fee will be billed.
                """

            amount: Amount
            """
            The amount billed for this fee.
            """
            bill_at: BillAt
            """
            When this fee will be billed. Always contains a concrete timestamp.
            """
            id: str
            """
            The ID of the one-time fee.
            """
            lookup_key: Optional[str]
            """
            The user-provided lookup key.
            """
            product: str
            """
            The ID of the v1 Product for this fee.
            """
            _inner_class_types = {"bill_at": BillAt}

        data: List[Data]
        """
        The one-time fees for this page.
        """
        _inner_class_types = {"data": Data}

    class PricingLines(StripeObject):
        class Data(StripeObject):
            class EndsAt(StripeObject):
                timestamp: str
                """
                The timestamp when the item ends.
                """

            class Pricing(StripeObject):
                class PriceDetails(StripeObject):
                    class PricingOverrides(StripeObject):
                        class Data(StripeObject):
                            class EndsAt(StripeObject):
                                timestamp: str
                                """
                                The timestamp when the item ends.
                                """

                            class OverwritePrice(StripeObject):
                                class Tier(StripeObject):
                                    flat_amount: Optional[str]
                                    """
                                    Price for the entire tier, represented as a decimal string in minor currency units.
                                    """
                                    unit_amount: Optional[str]
                                    """
                                    Per-unit price for units included in this tier, represented as a decimal string in minor currency units.
                                    """
                                    up_to_decimal: Optional[Decimal]
                                    """
                                    Up to and including this quantity will be contained in the tier.
                                    """
                                    up_to_inf: Optional[Literal["inf"]]
                                    """
                                    No upper bound to this tier.
                                    """
                                    _field_encodings = {
                                        "up_to_decimal": "decimal_string",
                                    }

                                tiering_mode: Optional[
                                    Literal["graduated", "volume"]
                                ]
                                """
                                Defines whether the tiered price should be graduated or volume-based.
                                """
                                tiers: List[Tier]
                                """
                                Each element represents a pricing tier.
                                """
                                unit_amount: Optional[str]
                                """
                                The per-unit amount to be charged, represented as a decimal string in minor currency units.
                                """
                                _inner_class_types = {"tiers": Tier}

                            class StartsAt(StripeObject):
                                timestamp: str
                                """
                                The timestamp when the item starts.
                                """

                            ends_at: EndsAt
                            """
                            Resolved timestamp when this override ends.
                            """
                            lookup_key: Optional[str]
                            """
                            The user-provided lookup key for this override.
                            """
                            overwrite_price: Optional[OverwritePrice]
                            """
                            Details for an overwrite_price override.
                            """
                            pricing_override: str
                            """
                            The ID of the pricing line override.
                            """
                            starts_at: StartsAt
                            """
                            Resolved timestamp when this override starts.
                            """
                            type: Literal["overwrite_price"]
                            """
                            The type of override.
                            """
                            _inner_class_types = {
                                "ends_at": EndsAt,
                                "overwrite_price": OverwritePrice,
                                "starts_at": StartsAt,
                            }

                        data: List[Data]
                        """
                        The pricing line overrides.
                        """
                        _inner_class_types = {"data": Data}

                    current_quantity: Decimal
                    """
                    The current quantity on this pricing line.
                    """
                    price: str
                    """
                    The ID of the V1 price.
                    """
                    pricing_overrides: Optional[PricingOverrides]
                    """
                    The overwrite_price overrides embedded directly on this pricing line.
                    """
                    _inner_class_types = {
                        "pricing_overrides": PricingOverrides
                    }
                    _field_encodings = {"current_quantity": "decimal_string"}

                price_details: Optional[PriceDetails]
                """
                V1 price details. Present when `type` is `price`.
                """
                type: Literal["price"]
                """
                The type of pricing.
                """
                _inner_class_types = {"price_details": PriceDetails}

            class StartsAt(StripeObject):
                timestamp: str
                """
                The timestamp when the item starts.
                """

            ends_at: EndsAt
            """
            Resolved timestamp when the pricing line ends.
            """
            id: str
            """
            The ID of the pricing line.
            """
            lookup_key: Optional[str]
            """
            The user-provided lookup key for the pricing line.
            """
            metadata: Optional[UntypedStripeObject[str]]
            """
            Set of key-value pairs that you can attach to an object.
            """
            pricing: Pricing
            """
            The pricing configuration for the pricing line.
            """
            starts_at: StartsAt
            """
            Resolved timestamp when the pricing line starts.
            """
            _inner_class_types = {
                "ends_at": EndsAt,
                "pricing": Pricing,
                "starts_at": StartsAt,
            }

        data: List[Data]
        """
        The pricing lines for this page.
        """
        _inner_class_types = {"data": Data}

    class PricingOverrides(StripeObject):
        class Data(StripeObject):
            class EndsAt(StripeObject):
                timestamp: str
                """
                The timestamp when the item ends.
                """

            class Multiplier(StripeObject):
                class Criterion(StripeObject):
                    pricing_line_ids: Optional[List[str]]
                    """
                    Filter by pricing line IDs.
                    """
                    pricing_line_lookup_keys: Optional[List[str]]
                    """
                    Filter by pricing line lookup keys.
                    """
                    type: Literal["exclude", "include"]
                    """
                    Whether to include or exclude items matching these criteria.
                    """

                criteria: List[Criterion]
                """
                Criteria determining which rates the multiplier applies to.
                """
                factor: str
                """
                The multiplier factor, represented as a decimal string. e.g. "0.8" for a 20% reduction.
                """
                _inner_class_types = {"criteria": Criterion}

            class StartsAt(StripeObject):
                timestamp: str
                """
                The timestamp when the item starts.
                """

            ends_at: EndsAt
            """
            Resolved timestamp when the pricing override ends.
            """
            id: str
            """
            The ID of the pricing override.
            """
            lookup_key: Optional[str]
            """
            The user-provided lookup key for the pricing override.
            """
            multiplier: Optional[Multiplier]
            """
            Details for a multiplier override.
            """
            priority: int
            """
            The priority of this override relative to others. Lower number = higher priority.
            """
            starts_at: StartsAt
            """
            Resolved timestamp when the pricing override starts.
            """
            type: Literal["multiplier"]
            """
            The type of pricing override.
            """
            _inner_class_types = {
                "ends_at": EndsAt,
                "multiplier": Multiplier,
                "starts_at": StartsAt,
            }

        data: List[Data]
        """
        The pricing overrides for this page.
        """
        _inner_class_types = {"data": Data}

    class StatusDetails(StripeObject):
        class Active(StripeObject):
            activated_at: str
            """
            The timestamp when the contract was activated.
            """

        class Canceled(StripeObject):
            canceled_at: str
            """
            The timestamp when the contract was canceled.
            """

        active: Optional[Active]
        """
        Details of the active contract status.
        """
        canceled: Optional[Canceled]
        """
        Details of the canceled contract status.
        """
        _inner_class_types = {"active": Active, "canceled": Canceled}

    billing_cycle_anchor: Optional[BillingCycleAnchor]
    """
    The billing cycle anchor for the contract.
    """
    billing_settings: Optional[BillingSettings]
    """
    The billing settings for the contract.
    """
    contract_number: str
    """
    A unique user-provided contract number e.g. C-2026-0001.
    """
    created: str
    """
    Timestamp of when the object was created.
    """
    currency: str
    """
    The currency of the contract.
    """
    customer: str
    """
    The ID of the customer associated with the contract.
    """
    id: str
    """
    The ID of the contract object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of key-value pairs that you can attach to an object.
    """
    object: Literal["v2.billing.contract"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    one_time_fees: Optional[OneTimeFees]
    """
    The one-time fees of the contract. Only populated when `one_time_fees` is passed in the `include` parameter.
    """
    pricing_lines: Optional[PricingLines]
    """
    The pricing lines of the contract. Only populated when `pricing_lines` is passed in the `include` parameter.
    """
    pricing_overrides: Optional[PricingOverrides]
    """
    The pricing overrides of the contract. Only populated when `pricing_overrides` is passed in the `include` parameter.
    """
    status: Literal["active", "canceled", "draft", "ended"]
    """
    The current status of the contract.
    """
    status_details: StatusDetails
    """
    Information about the contract status transitions.
    """
    _inner_class_types = {
        "billing_cycle_anchor": BillingCycleAnchor,
        "billing_settings": BillingSettings,
        "one_time_fees": OneTimeFees,
        "pricing_lines": PricingLines,
        "pricing_overrides": PricingOverrides,
        "status_details": StatusDetails,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.billing import analytics as analytics
    from stripe.params.billing._alert_activate_params import (
        AlertActivateParams as AlertActivateParams,
    )
    from stripe.params.billing._alert_archive_params import (
        AlertArchiveParams as AlertArchiveParams,
    )
    from stripe.params.billing._alert_create_params import (
        AlertCreateParams as AlertCreateParams,
        AlertCreateParamsCreditBalanceThreshold as AlertCreateParamsCreditBalanceThreshold,
        AlertCreateParamsCreditBalanceThresholdFilter as AlertCreateParamsCreditBalanceThresholdFilter,
        AlertCreateParamsCreditBalanceThresholdFilterCreditGrants as AlertCreateParamsCreditBalanceThresholdFilterCreditGrants,
        AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfig as AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfig,
        AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScope as AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScope,
        AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScopeBillableItem as AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScopeBillableItem,
        AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScopePrice as AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScopePrice,
        AlertCreateParamsCreditBalanceThresholdLte as AlertCreateParamsCreditBalanceThresholdLte,
        AlertCreateParamsCreditBalanceThresholdLteCustomPricingUnit as AlertCreateParamsCreditBalanceThresholdLteCustomPricingUnit,
        AlertCreateParamsCreditBalanceThresholdLteMonetary as AlertCreateParamsCreditBalanceThresholdLteMonetary,
        AlertCreateParamsSpendThreshold as AlertCreateParamsSpendThreshold,
        AlertCreateParamsSpendThresholdFilters as AlertCreateParamsSpendThresholdFilters,
        AlertCreateParamsSpendThresholdGte as AlertCreateParamsSpendThresholdGte,
        AlertCreateParamsSpendThresholdGteAmount as AlertCreateParamsSpendThresholdGteAmount,
        AlertCreateParamsSpendThresholdGteCustomPricingUnit as AlertCreateParamsSpendThresholdGteCustomPricingUnit,
        AlertCreateParamsUsageThreshold as AlertCreateParamsUsageThreshold,
        AlertCreateParamsUsageThresholdFilter as AlertCreateParamsUsageThresholdFilter,
    )
    from stripe.params.billing._alert_deactivate_params import (
        AlertDeactivateParams as AlertDeactivateParams,
    )
    from stripe.params.billing._alert_list_params import (
        AlertListParams as AlertListParams,
    )
    from stripe.params.billing._alert_retrieve_params import (
        AlertRetrieveParams as AlertRetrieveParams,
    )
    from stripe.params.billing._credit_balance_summary_retrieve_params import (
        CreditBalanceSummaryRetrieveParams as CreditBalanceSummaryRetrieveParams,
        CreditBalanceSummaryRetrieveParamsFilter as CreditBalanceSummaryRetrieveParamsFilter,
        CreditBalanceSummaryRetrieveParamsFilterApplicabilityScope as CreditBalanceSummaryRetrieveParamsFilterApplicabilityScope,
        CreditBalanceSummaryRetrieveParamsFilterApplicabilityScopeBillableItem as CreditBalanceSummaryRetrieveParamsFilterApplicabilityScopeBillableItem,
        CreditBalanceSummaryRetrieveParamsFilterApplicabilityScopePrice as CreditBalanceSummaryRetrieveParamsFilterApplicabilityScopePrice,
    )
    from stripe.params.billing._credit_balance_transaction_list_params import (
        CreditBalanceTransactionListParams as CreditBalanceTransactionListParams,
    )
    from stripe.params.billing._credit_balance_transaction_retrieve_params import (
        CreditBalanceTransactionRetrieveParams as CreditBalanceTransactionRetrieveParams,
    )
    from stripe.params.billing._credit_grant_create_params import (
        CreditGrantCreateParams as CreditGrantCreateParams,
        CreditGrantCreateParamsAmount as CreditGrantCreateParamsAmount,
        CreditGrantCreateParamsAmountCustomPricingUnit as CreditGrantCreateParamsAmountCustomPricingUnit,
        CreditGrantCreateParamsAmountMonetary as CreditGrantCreateParamsAmountMonetary,
        CreditGrantCreateParamsApplicabilityConfig as CreditGrantCreateParamsApplicabilityConfig,
        CreditGrantCreateParamsApplicabilityConfigScope as CreditGrantCreateParamsApplicabilityConfigScope,
        CreditGrantCreateParamsApplicabilityConfigScopeBillableItem as CreditGrantCreateParamsApplicabilityConfigScopeBillableItem,
        CreditGrantCreateParamsApplicabilityConfigScopePrice as CreditGrantCreateParamsApplicabilityConfigScopePrice,
    )
    from stripe.params.billing._credit_grant_expire_params import (
        CreditGrantExpireParams as CreditGrantExpireParams,
    )
    from stripe.params.billing._credit_grant_list_params import (
        CreditGrantListParams as CreditGrantListParams,
    )
    from stripe.params.billing._credit_grant_modify_params import (
        CreditGrantModifyParams as CreditGrantModifyParams,
    )
    from stripe.params.billing._credit_grant_retrieve_params import (
        CreditGrantRetrieveParams as CreditGrantRetrieveParams,
    )
    from stripe.params.billing._credit_grant_update_params import (
        CreditGrantUpdateParams as CreditGrantUpdateParams,
    )
    from stripe.params.billing._credit_grant_void_grant_params import (
        CreditGrantVoidGrantParams as CreditGrantVoidGrantParams,
    )
    from stripe.params.billing._meter_create_params import (
        MeterCreateParams as MeterCreateParams,
        MeterCreateParamsCustomerMapping as MeterCreateParamsCustomerMapping,
        MeterCreateParamsDefaultAggregation as MeterCreateParamsDefaultAggregation,
        MeterCreateParamsValueSettings as MeterCreateParamsValueSettings,
    )
    from stripe.params.billing._meter_deactivate_params import (
        MeterDeactivateParams as MeterDeactivateParams,
    )
    from stripe.params.billing._meter_event_adjustment_create_params import (
        MeterEventAdjustmentCreateParams as MeterEventAdjustmentCreateParams,
        MeterEventAdjustmentCreateParamsCancel as MeterEventAdjustmentCreateParamsCancel,
    )
    from stripe.params.billing._meter_event_create_params import (
        MeterEventCreateParams as MeterEventCreateParams,
    )
    from stripe.params.billing._meter_event_summary_list_params import (
        MeterEventSummaryListParams as MeterEventSummaryListParams,
    )
    from stripe.params.billing._meter_list_event_summaries_params import (
        MeterListEventSummariesParams as MeterListEventSummariesParams,
    )
    from stripe.params.billing._meter_list_params import (
        MeterListParams as MeterListParams,
    )
    from stripe.params.billing._meter_modify_params import (
        MeterModifyParams as MeterModifyParams,
    )
    from stripe.params.billing._meter_reactivate_params import (
        MeterReactivateParams as MeterReactivateParams,
    )
    from stripe.params.billing._meter_retrieve_params import (
        MeterRetrieveParams as MeterRetrieveParams,
    )
    from stripe.params.billing._meter_update_params import (
        MeterUpdateParams as MeterUpdateParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "analytics": ("stripe.params.billing.analytics", True),
    "AlertActivateParams": (
        "stripe.params.billing._alert_activate_params",
        False,
    ),
    "AlertArchiveParams": (
        "stripe.params.billing._alert_archive_params",
        False,
    ),
    "AlertCreateParams": ("stripe.params.billing._alert_create_params", False),
    "AlertCreateParamsCreditBalanceThreshold": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsCreditBalanceThresholdFilter": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsCreditBalanceThresholdFilterCreditGrants": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfig": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScope": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScopeBillableItem": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScopePrice": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsCreditBalanceThresholdLte": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsCreditBalanceThresholdLteCustomPricingUnit": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsCreditBalanceThresholdLteMonetary": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsSpendThreshold": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsSpendThresholdFilters": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsSpendThresholdGte": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsSpendThresholdGteAmount": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsSpendThresholdGteCustomPricingUnit": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsUsageThreshold": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertCreateParamsUsageThresholdFilter": (
        "stripe.params.billing._alert_create_params",
        False,
    ),
    "AlertDeactivateParams": (
        "stripe.params.billing._alert_deactivate_params",
        False,
    ),
    "AlertListParams": ("stripe.params.billing._alert_list_params", False),
    "AlertRetrieveParams": (
        "stripe.params.billing._alert_retrieve_params",
        False,
    ),
    "CreditBalanceSummaryRetrieveParams": (
        "stripe.params.billing._credit_balance_summary_retrieve_params",
        False,
    ),
    "CreditBalanceSummaryRetrieveParamsFilter": (
        "stripe.params.billing._credit_balance_summary_retrieve_params",
        False,
    ),
    "CreditBalanceSummaryRetrieveParamsFilterApplicabilityScope": (
        "stripe.params.billing._credit_balance_summary_retrieve_params",
        False,
    ),
    "CreditBalanceSummaryRetrieveParamsFilterApplicabilityScopeBillableItem": (
        "stripe.params.billing._credit_balance_summary_retrieve_params",
        False,
    ),
    "CreditBalanceSummaryRetrieveParamsFilterApplicabilityScopePrice": (
        "stripe.params.billing._credit_balance_summary_retrieve_params",
        False,
    ),
    "CreditBalanceTransactionListParams": (
        "stripe.params.billing._credit_balance_transaction_list_params",
        False,
    ),
    "CreditBalanceTransactionRetrieveParams": (
        "stripe.params.billing._credit_balance_transaction_retrieve_params",
        False,
    ),
    "CreditGrantCreateParams": (
        "stripe.params.billing._credit_grant_create_params",
        False,
    ),
    "CreditGrantCreateParamsAmount": (
        "stripe.params.billing._credit_grant_create_params",
        False,
    ),
    "CreditGrantCreateParamsAmountCustomPricingUnit": (
        "stripe.params.billing._credit_grant_create_params",
        False,
    ),
    "CreditGrantCreateParamsAmountMonetary": (
        "stripe.params.billing._credit_grant_create_params",
        False,
    ),
    "CreditGrantCreateParamsApplicabilityConfig": (
        "stripe.params.billing._credit_grant_create_params",
        False,
    ),
    "CreditGrantCreateParamsApplicabilityConfigScope": (
        "stripe.params.billing._credit_grant_create_params",
        False,
    ),
    "CreditGrantCreateParamsApplicabilityConfigScopeBillableItem": (
        "stripe.params.billing._credit_grant_create_params",
        False,
    ),
    "CreditGrantCreateParamsApplicabilityConfigScopePrice": (
        "stripe.params.billing._credit_grant_create_params",
        False,
    ),
    "CreditGrantExpireParams": (
        "stripe.params.billing._credit_grant_expire_params",
        False,
    ),
    "CreditGrantListParams": (
        "stripe.params.billing._credit_grant_list_params",
        False,
    ),
    "CreditGrantModifyParams": (
        "stripe.params.billing._credit_grant_modify_params",
        False,
    ),
    "CreditGrantRetrieveParams": (
        "stripe.params.billing._credit_grant_retrieve_params",
        False,
    ),
    "CreditGrantUpdateParams": (
        "stripe.params.billing._credit_grant_update_params",
        False,
    ),
    "CreditGrantVoidGrantParams": (
        "stripe.params.billing._credit_grant_void_grant_params",
        False,
    ),
    "MeterCreateParams": ("stripe.params.billing._meter_create_params", False),
    "MeterCreateParamsCustomerMapping": (
        "stripe.params.billing._meter_create_params",
        False,
    ),
    "MeterCreateParamsDefaultAggregation": (
        "stripe.params.billing._meter_create_params",
        False,
    ),
    "MeterCreateParamsValueSettings": (
        "stripe.params.billing._meter_create_params",
        False,
    ),
    "MeterDeactivateParams": (
        "stripe.params.billing._meter_deactivate_params",
        False,
    ),
    "MeterEventAdjustmentCreateParams": (
        "stripe.params.billing._meter_event_adjustment_create_params",
        False,
    ),
    "MeterEventAdjustmentCreateParamsCancel": (
        "stripe.params.billing._meter_event_adjustment_create_params",
        False,
    ),
    "MeterEventCreateParams": (
        "stripe.params.billing._meter_event_create_params",
        False,
    ),
    "MeterEventSummaryListParams": (
        "stripe.params.billing._meter_event_summary_list_params",
        False,
    ),
    "MeterListEventSummariesParams": (
        "stripe.params.billing._meter_list_event_summaries_params",
        False,
    ),
    "MeterListParams": ("stripe.params.billing._meter_list_params", False),
    "MeterModifyParams": ("stripe.params.billing._meter_modify_params", False),
    "MeterReactivateParams": (
        "stripe.params.billing._meter_reactivate_params",
        False,
    ),
    "MeterRetrieveParams": (
        "stripe.params.billing._meter_retrieve_params",
        False,
    ),
    "MeterUpdateParams": ("stripe.params.billing._meter_update_params", False),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import NotRequired, TypedDict


class DisputeSimulateNetworkLifecyclePreArbitrationResponseParams(TypedDict):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    merchant_evidence_files: "DisputeSimulateNetworkLifecyclePreArbitrationResponseParamsMerchantEvidenceFiles"
    """
    Controls the acquiring merchant's simulated submitted evidence files for the pre-arbitration response stage.
    """


class DisputeSimulateNetworkLifecyclePreArbitrationResponseParamsMerchantEvidenceFiles(
    TypedDict,
):
    number_to_generate: int
    """
    How many simulated merchant evidence file tokens to attach (between 1 and 12).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.delegated_checkout._order import Order
    from stripe.params.delegated_checkout._order_retrieve_params import (
        OrderRetrieveParams,
    )


class OrderService(StripeService):
    def retrieve(
        self,
        order_id: str,
        params: Optional["OrderRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Order":
        """
        Retrieves a delegated checkout order.
        """
        return cast(
            "Order",
            self._request(
                "get",
                "/v1/delegated_checkout/orders/{order_id}".format(
                    order_id=sanitize_id(order_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        order_id: str,
        params: Optional["OrderRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Order":
        """
        Retrieves a delegated checkout order.
        """
        return cast(
            "Order",
            await self._request_async(
                "get",
                "/v1/delegated_checkout/orders/{order_id}".format(
                    order_id=sanitize_id(order_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )
